from tests.base_test import BaseTest
from disco.models import JobDetails


class TestJobDetailsModel(BaseTest):

    def test_job_create(self):

        job_id = self.random_str('job-id')
        job_name = self.random_str('job-name')
        job_status = self.random_str('Working')

        tasks_summary = {
            'timeout': self.random_int(),
            'queued': self.random_int(),
            'running': self.random_int(),
            'failed': self.random_int(),
            'success': self.random_int(),
            'all': self.random_int()
        }

        job_data = {
            'id': job_id,
            'request': {'meta': {'name': job_name}},
            'status': job_status,
            'tasksSummary': tasks_summary
        }

        job_details = JobDetails(job_data)
        assert job_details.id == job_id
        assert job_details.name == job_name
        assert job_details.status == job_status
        assert job_details.tasks_summary.timeout == tasks_summary['timeout']
        assert job_details.tasks_summary.queued == tasks_summary['queued']
        assert job_details.tasks_summary.running == tasks_summary['running']
        assert job_details.tasks_summary.failed == tasks_summary['failed']
        assert job_details.tasks_summary.success == tasks_summary['success']
        assert job_details.tasks_summary.all == tasks_summary['all']

    def test_job_create_without_tasks_summary(self):

        job_id = self.random_str('job-id')
        job_name = self.random_str('job-name')
        job_status = self.random_str('Working')

        job_data = {
            'id': job_id,
            'request': {'meta': {'name': job_name}},
            'status': job_status,
        }

        job_details = JobDetails(job_data)
        assert job_details.id == job_id
        assert job_details.name == job_name
        assert job_details.status == job_status
        assert job_details.tasks_summary is None



from .job_details import JobDetails
from .task import Task
from .cluster_details import ClusterDetails
from .repository_details import RepositoryDetails
from .docker_image_details import  DockerImageDetails



#  Copyright (c) 2019 Samsung, Inc. and its affiliates.
#
#  This source code is licensed under the RESTRICTED license found in the
#  LICENSE file in the root directory of this source tree.

import os
import shutil
import sys
import textwrap
import pytest
import mock
import virtualenv
from tempfile import TemporaryDirectory
import pathlib
import subprocess
import disco
from disco.core.constants import JobStatus
from tests.base_test import BaseTest
from tests.live import env
from .env import LIVE_TESTS_TIMEOUT_SECONDS


@pytest.mark.skipif(env.skip, reason=env.reason)
class TestTasksLive(BaseTest):

    def setup_class(self):
        BaseTest.disable_progress_bar()

    def test_files(self):

        expected_output = 'All done!'

        task_content = textwrap.dedent('''
            import os
            import sys
            import time
            os.makedirs('run-result/foo/bar/baz')
            open('run-result/file1', 'w').write('tip\\n'
                                     'top')
            open('run-result/foo/file2', 'w')
            glissando = (bytes(range(256)) if sys.version_info[0] == 3 else
                         b''.join(chr(i) for i in range(256)))
            open('run-result/foo/bar/baz/file3', 'wb').write(glissando)
            time.sleep(3)
            print('All done!')
        ''')
        script_file_id = disco.upload_file('woof.py', task_content)

        job = disco.Job.create(script_file_id)
        job.start()

        print(f'Waiting for job {job.job_id} to finish...')

        job.wait_for_status(JobStatus.done, interval=10, timeout=LIVE_TESTS_TIMEOUT_SECONDS)

        print(f'job {job.job_id} finished!')
        print('Waiting for the jobs results')

        (task_result,) = job.get_results(block=True)

        print('Finished getting the results')

        task_result_output = self._stdout_to_output(task_result.stdout)
        assert task_result_output == expected_output

        glissando = (bytes(range(256)) if sys.version_info[0] == 3 else
                     b''.join(chr(i) for i in range(256)))
        assert ('foo/bar/baz/file3', glissando) in task_result.raw_result

        with TemporaryDirectory(prefix='test_disco_') as temp_folder:
            temp_folder = pathlib.Path(str(temp_folder))
            task_result.write_files(temp_folder)
            current_paths = list(temp_folder.rglob('*'))

            assert len(current_paths) == 8

            (output_path,) = (path for path in current_paths if path == temp_folder / 'IqoqoTask.stdout.0.txt')
            current_paths.remove(output_path)

            output_path_text = self._stdout_to_output(output_path.read_text())

            assert output_path_text == expected_output

            (file1_path,) = (path for path in current_paths if path == temp_folder / 'file1')
            current_paths.remove(file1_path)
            assert file1_path.read_text() == ('tip\n'
                                              'top')

            (file2_path,) = (path for path in current_paths if path == temp_folder / 'foo' / 'file2')
            current_paths.remove(file2_path)
            assert file2_path.read_text() == ''

            (file3_path,) = (path for path in current_paths if path == temp_folder / 'foo' / 'bar' / 'baz' / 'file3')
            current_paths.remove(file3_path)
            assert file3_path.read_bytes() == glissando

            folder_paths = [path for path in current_paths if path.name in ('foo', 'bar', 'baz')]
            for folder_path in folder_paths:
                assert folder_path.is_dir()
                current_paths.remove(folder_path)

            assert len(current_paths) == 1
            (meta_stderr_path,) = current_paths
            assert 'stderr' in meta_stderr_path.name
            assert meta_stderr_path.is_file()

    def test_multiple_tasks(self):
        task_content = textwrap.dedent('''
        import sys
        import time
        content = open(sys.argv[1]).read()
        print('%s' % content)
        time.sleep(3)
        ''')

        number_of_inputs = 30

        expected_outputs = [self.random_str(prefix=f"output{i}-") for i in range(number_of_inputs)]

        print(f'Uploading input files...')

        input_file_ids = [disco.upload_file(f'input_{i}.txt', expected_outputs[i]) for i in range(number_of_inputs)]
        script_file_id = disco.upload_file('meow.py', task_content)

        job = disco.Job.create(
            script_file_id,
            input_file_ids=input_file_ids
        )

        job.start()

        print(f'Waiting for job {job.job_id} to finish...')

        job.wait_for_status(JobStatus.done, interval=10, timeout=LIVE_TESTS_TIMEOUT_SECONDS)

        print(f'job {job.job_id} finished!')
        print('Waiting for the jobs results')

        task_results = job.get_results(block=True)

        print('Finished getting the results')

        assert len(expected_outputs) == len(task_results), 'expected output length matches the task results length'

        # Verify that the result matches the output
        task_result_outputs = [self._stdout_to_output(task_result.stdout) for task_result in task_results]

        assert task_result_outputs == expected_outputs, 'task result outputs matches the expected output'

    def test_running_in_venv(self):

        expected_output = 'All done!'

        venv_temp_dir = self._create_and_activate_venv()
        venv_pip_path = os.path.join(venv_temp_dir, "bin", "pip")
        venv_python_path = os.path.join(venv_temp_dir, "bin", "python")

        install_return_code = subprocess.call([venv_pip_path, "install", 'setuptools==42.0.2'])
        assert install_return_code == 0

        task_content = textwrap.dedent('''
        from setuptools import setup
        print('All done!')
        ''')

        script_file_id = disco.upload_file('code-with-package.py', task_content)

        # Ensure we use the correct python path when doing `pip freeze` to get the venv requirements
        with mock.patch('sys.executable', venv_python_path):
            job = disco.Job.create(script_file_id)

        job.start()

        print(f'Waiting for job {job.job_id} to finish...')

        job.wait_for_status(JobStatus.done, interval=10, timeout=LIVE_TESTS_TIMEOUT_SECONDS)

        print(f'job {job.job_id} finished!')
        print('Waiting for the jobs results')

        task_results = job.get_results(block=True)

        print('Finished getting the results')

        task_result_output = self._stdout_to_output(task_results[0].stdout)

        assert task_result_output == expected_output

        self._delete_venv()

    def test_get_tasks(self):
        task_content = textwrap.dedent('''
        import sys
        content = open(sys.argv[1]).read()
        print('foo %s' % content)
        ''')

        script_file_id = disco.upload_file('meow.py', task_content)

        job = disco.Job.create(script_file_id)

        print(f'Waiting for tasks of job {job.job_id}')

        job_tasks = job.get_tasks()

        print('Finished getting the tasks')

        assert isinstance(job_tasks, list)
        assert len(job_tasks) == 1

        task = job_tasks[0]

        assert task.id is not None
        assert task.status is not None
        assert task.duration >= 0

    @staticmethod
    def _stdout_to_output(stdout):
        stdout_str = stdout

        if isinstance(stdout, bytes):
            stdout_str = stdout.decode("utf-8")

        output_lines = stdout_str.strip().split('\n')
        return output_lines[-1]

    @staticmethod
    def _venv_temp_dir():
        return os.path.join(os.path.expanduser("~"), ".venv-test-temp")

    @classmethod
    def _delete_venv(cls):
        try:
            shutil.rmtree(cls._venv_temp_dir())
        except OSError:
            return

    @classmethod
    def _create_and_activate_venv(cls):
        venv_temp_dir = cls._venv_temp_dir()

        cls._delete_venv()

        virtualenv.create_environment(venv_temp_dir)
        path = os.path.join(venv_temp_dir, "bin", "activate_this.py")
        exec(open(path).read(), {'__file__': path})

        return venv_temp_dir



#  Copyright (c) 2019 Samsung, Inc. and its affiliates.
#
#  This source code is licensed under the RESTRICTED license found in the
#  LICENSE file in the root directory of this source tree.

import os
from disco.core import constants

skip = 'DISCO_LIVE_TESTS' not in os.environ
reason = ('Please pass an environment variable DISCO_LIVE_TESTS if you want'
          ' to run the integration tests on a live environment. Don\'t forget'
          ' to set environment variables for connecting to Dis.co servers')

LIVE_TESTS_TIMEOUT_SECONDS = 9 * constants.MINUTE



"""
Authorisation class for DISCO
used as singleton (`disco.gql.authentication`)
"""
import requests

from disco.core.exceptions import DiscoException, InvalidCredentials
from .query import Query, RestQuery
from ..core import exceptions, Credentials


def _handle_auth_errors(src_fn):
    #Wraps function and handles authentication errors.

    def _try(*args, **kwargs):
        last_error = None
        for _ in range(Authentication.MAX_AUTH_RETRIES):
            try:
                return src_fn(*args, **kwargs)
            except requests.HTTPError as http_error:
                status = http_error.response.status_code
                if status in Authentication.AuthenticationErrorStatusCodes:
                    last_error = http_error
                    Authentication.refresh()

                else:
                    raise
        _raise_errors(last_error)

    return _try


def _raise_errors(error: requests.HTTPError):
    #Turns standard http errors into `exceptions.RequestException`.
    response = error.response
    if response.content:
        try:
            response_json = response.json()
            errors = response_json.get('errors', [])
            if errors:
                errors_arr = [e['message'] for e in errors]
                raise exceptions.RequestException(errors_arr)
        except ValueError:
            raise exceptions.RequestException([response.content])
    errors_arr = [f"({response.status_code}) {response.reason}"]
    raise exceptions.RequestException(errors_arr)


class Authentication:
    """Class for handling authenticated communication to DISCO backend.
    Provides easy way to run `query`, `rest` and `stream` with authentication
    and token refreshes.
    """
    MAX_AUTH_RETRIES = 3
    AuthenticationErrorStatusCodes = (401, 403, 422)

    @classmethod
    @_handle_auth_errors
    def query(cls, *args, **kwargs):
        """Executes Authenticated GQL Query.

        Args:
            *args:
            **kwargs:

        Returns:
        """
        query = Query(*args, **kwargs)
        query.cookies = {'jwtToken': cls.get_token()}
        return query.send()

    @classmethod
    @_handle_auth_errors
    def rest(cls, *args, **kwargs):
        """Executes Authenticated Rest Query.

        Args:
            *args:
            **kwargs:

        Returns:
        """
        rest = RestQuery(*args, **kwargs)
        rest.cookies = {'jwtToken': cls.get_token()}
        return rest.send()

    @classmethod
    @_handle_auth_errors
    def stream(cls, *args, **kwargs):
        """Streams Authenticated Rest Query.

        Args:
            *args:
            **kwargs:

        Returns:
        """
        stream = RestQuery(*args, **kwargs)
        stream.cookies = {'jwtToken': cls.get_token()}
        return stream.stream()

    @classmethod
    def get_token(cls):
        """Gets the current `JWT` token. Fetches it if not present.

        Returns:
            The current `JWT` token.
        """
        if Credentials.token is None:
            cls._authenticate()
        return Credentials.token

    @classmethod
    def get_current_user(cls):
        """Gets the current username. Fetches it if not present.

        Returns:
            Current username.
        """
        if Credentials.username is None:
            cls._fetch_current_user()
        return Credentials.username

    @classmethod
    def get_current_user_id(cls):
        """Gets the id of the current user . Fetches it if not present.

        Returns:
            Current user id.
        """
        if Credentials.userid is None:
            cls._fetch_current_user()
        return Credentials.userid

    @classmethod
    def get_credentials(cls):
        """Loads and returns credentials for DISCO authentication.

        Returns:
        """
        if not Credentials.has_credentials():
            Credentials.credentials = Credentials.from_config()
        return Credentials.credentials

    @classmethod
    def set_credentials(cls, *args, **kwargs):
        """Updates `Credentials`. See: `Credentials.update`.

        Args:
            *args:
            **kwargs:

        Returns:
        """
        return Credentials.update(*args, **kwargs)

    @classmethod
    def _authenticate(cls):
        email, password = cls.get_credentials()
        token = cls._login_request(email, password)
        Credentials.token = token
        Credentials.username = None
        Credentials.userid = None

    @classmethod
    def _fetch_current_user(cls):

        response = cls.query('fetchCurrentUser')

        profiles = response['fetchUser']['profiles']
        if not profiles:
            raise exceptions.NoValidProfileException()

        Credentials.username = profiles[0]['id']
        Credentials.userid = response['fetchUser']['id']

    @classmethod
    def refresh(cls):
        """Refreshes the token against DISCO servers."""
        Credentials.token = None
        Credentials.username = None
        Credentials.userid = None
        cls._authenticate()

    @classmethod
    def _login_request(cls, email, password):
        try:
            response = Query('login', login=email, password=password).send()
            return response['login']['token']
        except exceptions.GraphQLRequestException as ex:
            errors = ex.messages()
            if len(errors) == 0:
                raise DiscoException("Unknown error")
            if 'Email or password incorrect' in errors[0] or 'Unauthorized' in errors[0]:
                raise InvalidCredentials("Wrong email or password")
            raise DiscoException("Unknown error")

    @classmethod
    def login(cls, email, password):
        """Perform login action"""
        token = cls._login_request(email, password)

        Credentials.token = token
        cls.set_credentials(email, password, save_to_config=True)

    @classmethod
    def logout(cls):
        """Logs out of dis.co by cleaning token and credetials"""
        Credentials.reset()

    @classmethod
    def is_logged_in(cls):
        """Returns true if user is logged in"""
        try:
            user, _ = cls.get_credentials()
            return user is not None and user != ""
        except exceptions.NoCredentials:
            return False



from .base_model import BaseModel


class ClusterDetails(BaseModel):
    """
    Cluster Details
    """

    def __init__(self, data, is_default_cloud):
        super(ClusterDetails, self).__init__(data)
        self._is_default = is_default_cloud

    @property
    def is_default(self):
        """
        Is this the default cluster, is it???
        Returns: True if this is the default cluster

        """
        return self._is_default

    @property
    def id(self):
        """
        Returns:
            str
        """
        return self._data.get('id')

    @property
    def name(self):
        """
        Cluster name
        Returns (str): cluster name

        """
        return self._data.get('name')

    @property
    def type(self):
        """
        Cluster cloud type
        Returns (str): cluster cloud name

        """
        return self._data.get('type')

    @property
    def region(self):
        """
        Cluster region
        Returns (str): cluster region

        """
        return self._data.get('cluster', {}).get('region')

    @property
    def external_account_id(self):
        """
        External cloud account id
        Returns (str): external account id

        """
        return self._data.get('cluster', {}).get('externalAccountId')

    @property
    def bucket_name(self):
        """
        The name of the bucket for storage
        Returns (str): Cloud storage bucket name

        """
        return self._data.get('cluster', {}).get('bucketName')

    @property
    def is_last_connection_check_valid(self):
        """
        Did the last connection check pass
        Returns (boolean): true is last connection test passed

        """
        return self._data.get('isLastConnectionCheckValid')

    @property
    def is_active(self):
        """
        Is the cloud active or disabled
        Returns (boolean): True is the cloud is not disabled

        """
        return self._data.get('isActive')

    @property
    def cloud_offload_enabled(self):
        """
        For on-prem if cloud offload is enabled
        Returns (boolean): True if cloud offload is enabled

        """
        return self._data.get('enableCloudOffload')



#  Copyright (c) 2019 Samsung, Inc. and its affiliates.
#
#  This source code is licensed under the RESTRICTED license found in the
#  LICENSE file in the root directory of this source tree.

from .helper import add



from tests.base_test import BaseTest
from disco_cli import cli, setup_cli
from mock import patch
from click.testing import CliRunner
from .cli_test_utils import output_message_includes
from disco.models import DockerImageDetails

class TestDockerImage(BaseTest):

    def setup_class(self):
        setup_cli()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.DockerImage.list_docker_images')
    def test_list_docker_images(self, list_docker_images_mock, is_logged_in_mock):
        is_logged_in_mock.return_value = True
        list_docker_images = [
            DockerImageDetails({'id': '5e143698be608b000c6064b0', 'displayName': 'docker 1',
                                'repositoryType': 'dockerHub', 'path': 'path1', 'entryPoint': 'ep1'}),
            DockerImageDetails({'id': '5e143698be608b000c6064b1', 'displayName': 'docker 2',
                                'repositoryType': 'aws', 'path': 'path2', 'entryPoint': 'ep2'})
        ]

        list_docker_images_mock.return_value = list_docker_images
        runner = CliRunner()
        result = runner.invoke(cli, ['docker', 'list'])
        assert result.exit_code == 0
        list_docker_images_mock.assert_called()
        assert output_message_includes(result, '| 5e143698be608b000c6064b0 | docker 1 | dockerHub         | path1  | ep1           |')
        assert output_message_includes(result, '| 5e143698be608b000c6064b1 | docker 2 | aws               | path2  | ep2           |')

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.DockerImage.list_docker_images')
    def test_empty_list_docker_images(self, list_docker_images_mock, is_logged_in_mock):
        is_logged_in_mock.return_value = True
        list_docker_images = []

        list_docker_images_mock.return_value = list_docker_images
        runner = CliRunner()
        result = runner.invoke(cli, ['docker', 'list'])
        assert result.exit_code == 0
        list_docker_images_mock.assert_called()
        assert result.output == 'There are no set docker images for your account\n'




# -*- coding: utf-8 -*-
import contextlib
import math
import sys
import time

# pylint: skip-file


def _default_fp(fp):
    return fp or sys.stderr


def _dynamic_display(fp):
    return (hasattr(fp, 'isatty') and fp.isatty()) or 'ipykernel' in sys.modules


class _NopeProgressbar:
    def update(self, inc=1):
        pass

    def set_postfix_str(self, s='', refresh=True):
        pass


class _SimpleProgressbar:
    def __init__(self, target, width=30, interval=0.05, fp=None):
        self._target = target
        self._width = width
        self._interval = interval
        self._desc = None

        self.__fp = _default_fp(fp)

        self._seen_so_far = 0
        self._start = time.time()
        self._last_update = 0

    def __get_time_per_unit(self, current, now):
        return (now - self._start) / current if current else 0

    @classmethod
    def __get_eta_format(cls, eta):
        if eta > 3600:
            eta_format = ('%d:%02d:%02d' %
                          (eta // 3600, (eta % 3600) // 60, eta % 60))
        elif eta > 60:
            eta_format = '%d:%02d' % (eta // 60, eta % 60)
        else:
            eta_format = '%ds' % eta

        return eta_format

    def __get_eta_str(self, current, now):
        info = ' - %.0fs' % (now - self._start)

        time_per_unit = self.__get_time_per_unit(current, now)

        if current < self._target:
            eta = time_per_unit * (self._target - current)

            if self._desc is not None:
                info += ' - %s' % self._desc

            info += ' - ETA: %s' % self.__get_eta_format(eta)

        return info

    def _should_skip_draw(self, current, now):
        return now - self._last_update < self._interval and self._target is not None and current < self._target

    def _get_bar(self, current):
        num_digits = int(math.floor(math.log10(self._target))) + 1 if self._target > 0 else 1

        prog = float(current) / self._target if self._target > 0 else 1

        prog_width = int(self._width * prog) or 1

        bar_str = '%%%dd/%d [' % (num_digits, self._target)
        progress_bar = bar_str % current

        progress_bar += ('=' * (prog_width - 1))

        if current < self._target:
            progress_bar += '>'
        else:
            progress_bar += '='

        progress_bar += ('.' * (self._width - prog_width))
        progress_bar += ']'

        return progress_bar

    def _full_update(self, current):
        self._seen_so_far = current

        now = time.time()
        if self._should_skip_draw(current, now):
            return

        self.__fp.write('\n')

        progress_bar = self._get_bar(current)

        self.__fp.write(progress_bar)

        info = self.__get_eta_str(current, now)

        self.__fp.write(info)
        self.__fp.flush()

        self._last_update = now

    def update(self, inc=1):
        self._full_update(self._seen_so_far + inc)

    def set_postfix_str(self, desc='', refresh=True):
        self._desc = desc


def _default_interval(interval, def_interval):
    return interval if interval is not None else def_interval


@contextlib.contextmanager
def create_progress_bar(total, desc=None, disable=False, interval=None, fp=None, **kwargs):
    from tqdm import tqdm

    if disable:
        yield _NopeProgressbar()
        return

    fp = _default_fp(fp)

    if _dynamic_display(fp):
        default_dynamic_interval = 0.1

        unit = kwargs.get('unit', 'it')
        ncols = kwargs.get('ncols', 80)
        unit_scale = kwargs.get('unit_scale', False)

        with tqdm(total=total, desc=desc, unit=unit, ncols=ncols,
                  disable=None, unit_scale=unit_scale, file=fp,
                  mininterval=_default_interval(interval, default_dynamic_interval)) as progress_bar:
            yield progress_bar
    else:
        default_static_interval = 5

        progress_bar = _SimpleProgressbar(total, interval=_default_interval(interval, default_static_interval), fp=fp)
        progress_bar.update(0)
        yield progress_bar



from .base_model import BaseModel


class DockerImageDetails(BaseModel):
    """
    Docker image details.
    """

    @property
    def id(self):
        """
        Returns:
            str
        """
        return self._data.get('id')

    @property
    def name(self):
        """
        Returns:
            str
        """
        return self._data.get('displayName')

    @property
    def repository_type(self):
        """
        Returns:
            str
        """
        return self._data.get('repositoryType')

    @property
    def path(self):
        """
        Returns:
            str
        """
        return self._data.get('path')

    @property
    def entry_point(self):
        """
        Returns:
            str
        """
        return self._data.get('entryPoint')



#  Copyright (c) 2019 Samsung, Inc. and its affiliates.
#
#  This source code is licensed under the RESTRICTED license found in the
#  LICENSE file in the root directory of this source tree.

import warnings

import pytest


@pytest.fixture(scope='session', autouse=True)
def ignore_deprecation_warning(request):
    warnings.filterwarnings('ignore', category=DeprecationWarning)



"""
Tool for retriving GQL commands from files.
Autoloads the queries present in the dir as dictionary
"""
import os
from pathlib import Path


def _load_commands():
    _here = os.path.abspath(os.path.dirname(__file__))
    return {p.name[:-4]: p.read_text() for p in Path(_here).glob("*.gql")}


REGISTERED_COMMANDS = _load_commands()



"""
Constants and Enum used by the sdk
"""
import enum
import pathlib

REST_API_SUFFIX = '/api/v1'
DEFAULT_BASE_URL = 'https://app.dis.co'
DISCO_CONFIG_PATH = pathlib.Path.home() / '.disco_py'
ENV_VAR_BASE_URL_NAME = 'DISCO_BASE_URL'
ENV_VAR_EMAIL_NAME = 'DISCO_EMAIL'
ENV_VAR_PASSWORD_NAME = 'DISCO_PASSWORD'
ENV_VAR_DISABLE_PROGRESS_BAR = 'DISCO_DISABLE_PROGRESS_BAR'

SECOND = 1
MINUTE = 60 * SECOND
HOUR = 60 * MINUTE
DAY = 24 * HOUR


class JobStatus(enum.Enum):
    """Job statuses"""
    unknown = 'Unknown'
    listed = 'Listed'
    working = 'Working'
    stopped = 'Stopped'
    done = 'Done'
    failed = 'Failed'
    deleted = 'Deleted'


JOB_RUNNING_STATES = {JobStatus.listed, JobStatus.working}
JOB_TERMINAL_STATES = {x for x in JobStatus if x not in JOB_RUNNING_STATES}


class TaskStatus(enum.Enum):
    """Task statuses"""
    unknown = 'Unknown'
    listed = 'Listed'
    running = 'Running'
    stopped = 'Stopped'
    success = 'Success'
    failed = 'Failed'
    deleted = 'Deleted'



from .base_model import BaseModel


class RepositoryDetails(BaseModel):
    """
    Repository Details
    """

    @property
    def id(self):
        """
        Returns:
            str
        """
        return self._data.get('id')

    @property
    def name(self):
        """
        Returns:
            str
        """
        return self._data.get('displayName')

    @property
    def is_active(self):
        """
        Returns:
            str
        """
        return self._data.get('isActive')



from disco.base_controller import BaseController
from disco.models import DockerImageDetails


class DockerImage(BaseController):
    """
    Docker image methods
    """

    @classmethod
    def list_docker_images(cls, limit=None, next_=None):
        """Show a list of all the docker images of this user.

                Args:
                    limit (int): pagination limit
                    next_: pagination next

                Returns:
                    list(): List of the docker images of this user.
                """

        res = cls.query('getDockerImagesByOwnerId', limit=limit,
                        ownerId=cls.get_current_user(), next=next_)

        if res is None:
            return []

        docker_images = [DockerImageDetails(docker_image) for docker_image in res['getDockerImagesByOwnerId']]

        return docker_images



import click

from cli.command_utils import info_message, list_to_table
from disco import Repository

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

@click.group(context_settings=CONTEXT_SETTINGS)
def repository():
    """Manage repositories."""


@repository.command('list', context_settings=CONTEXT_SETTINGS, short_help="List of repositories for the current user.")
def list_repositories():
    """
    List of repositories for the current user.

    $ disco repository list
    """
    repositories = Repository.list_repositories()
    if len(repositories) > 0:
        _display_repositories(repositories)
    else:
        info_message("There are no set repositories for your account")

def _display_repositories(repositories_list):
    """
    Display repositories as table
    Args:
        repositories_list: list of repositories details

    Returns:
    """
    repository_table = [[rep.id, rep.name] for rep in repositories_list]
    info_message(list_to_table(repository_table, headers=["ID", "Name"]))



import errno
import glob
import os
import time
from pathlib import Path
import click

from disco import Job, upload_file, input_files_from_bucket
from disco.core.constants import JobStatus
from disco.core.exceptions import DiscoException, TimeOutError, GraphQLRequestException, \
    BucketPathsException, BucketPathsErrorTypes
from disco.models import JobDetails

from .command_utils import info_message, list_to_table, \
    error_message, success_message, verify_logged_in, exception_message, ExpandedPath
from .context_state import pass_state
from .common_options import common_options

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group('job', context_settings=CONTEXT_SETTINGS)
def job_commands():
    """Manage jobs."""


@job_commands.command('create', context_settings=CONTEXT_SETTINGS, short_help="Create a new job.")
@click.option('-s', '--script', help='Local path to the script file to run.',
              type=ExpandedPath(exists=True))
@click.option('-i', '--input', 'inputs',
              help='Local comma-separated path to the input file(s) for the script, (wildcards allowed).')
@click.option('-b', '--bucket', 'raw_bucket_paths',
              help="Comma-separated paths for a bucket in your cluster"
                   " to scan for the input files for the script."
                   " Only file and directory paths are allowed. Wildcards are not supported.")
@click.option('-c', '--constants', help='Local path to the constants file that will be sent to each task.')
@click.option('-n', '--name', help='Job name.', prompt="Please enter job name")
@click.option('-r', '--run', help='Create and run the job.', is_flag=True)
@click.option('-w', '--wait', help='Wait for the job to finish.', is_flag=True)
@click.option('-cit', '--instance-type',
              help="Cluster instance type, options are: -s (small - default), -m (meduim), -l (large).",
              type=click.Choice(['s', 'm', 'l']), default='s')
@click.option('-cid', '--cluster-id', help="Cluster id to run the job on. "
                                           "You can use the ‘cluster list’ command to view your clusters. "
                                           "In case no cluster-id specified, Dis.co runs the job "
                                           "on the default Dis.co cluster.")
@click.option('-rep', '--repository-id', help='ID of the repository in which the script file resides.')
@click.option('-f', '--file-path', 'script_file_path_in_repo', help='Path in the Git repository to the script file.')
@click.option('-did', '--docker-image-id', help='The id of the docker image to run the job in')  # pylint: disable=too-many-arguments,too-many-locals, too-many-branches, , too-many-return-statements
@click.option('--debug-mode', hidden=True, is_flag=True)
@click.option('--dont-generate-req-file', hidden=True, is_flag=True)
@common_options
@pass_state
def create_job(state, script, inputs, raw_bucket_paths, constants, name, run, wait, instance_type, cluster_id,  # pylint: disable=redefined-builtin
               repository_id, script_file_path_in_repo, docker_image_id, debug_mode, dont_generate_req_file):
    """
    Create a new job that can be launched immediately, or later.

    $ disco job create —n <job’s name> -s <script file path> -i <input file path> --run
    """
    if not verify_logged_in():
        return

    if script is None and repository_id is None:
        error_message("Please specify the script file using either `--script` "
                      "or `--repository-id` with `--file-path`")
        return
    if repository_id is not None and script_file_path_in_repo is None:
        error_message("Please specify the script file path using `--file-path`")
        return

    script_path = Path(script) if script is not None else Path(script_file_path_in_repo)
    if (script_path.suffix not in ['.sh', '.py']):
        error_message("Cannot use script file. Currently only Python and bash scripts are supported")
        return

    input_file_ids = []

    if raw_bucket_paths:
        bucket_input_file_ids, is_error = _get_bucket_input_file_ids(raw_bucket_paths, cluster_id)
        if is_error:
            return
        input_file_ids += bucket_input_file_ids

    try:
        input_paths = get_file_list(inputs) if inputs else []
        const_paths = get_file_list(constants) if constants else []
    except FileNotFoundError as ex:
        error_message(str(ex))
        return

    try:
        show_progress_bar = not state.quite_mode
        const_file_ids, local_input_file_ids = upload_job_files(input_paths, const_paths, cluster_id, show_progress_bar)
        input_file_ids += local_input_file_ids

        script_file_id = None if script is None else upload_job_script_file(script, cluster_id, show_progress_bar)
        created_job = Job.create(script_file_id=script_file_id, input_file_ids=input_file_ids,
                                 constants_file_ids=const_file_ids, job_name=name, cluster_instance_type=instance_type,
                                 cluster_id=cluster_id, script_repo_id=repository_id,
                                 script_file_path_in_repo=script_file_path_in_repo, auto_start=run,
                                 upload_requirements_file=not dont_generate_req_file, docker_image_id=docker_image_id)

    except GraphQLRequestException as ex:
        messages = list(map(lambda error: error['message'], ex.errors))
        if any(msg.find('Unable to find artifacts repository') >= 0 for msg in messages):
            error_message('Invalid repository id, repository does not exist (try `repository list` command)')
            return
        if any(msg.find('doesn\'t own the repository provided') >= 0 for msg in messages):
            error_message('You don`t have permission for this repository')
            return
        if any(msg.find('Unable to find docker image') >= 0 for msg in messages):
            error_message('Invalid docker image id, docker image does not exist')
            return
        if any(msg.find('Unauthorized') >= 0 for msg in messages):  # change backend message to more specific one
            error_message('You don`t have permission for this docker image')
            return
    except DiscoException as ex:
        if debug_mode:
            print(ex)
        exception_message(ex)
        return

    job_status = None
    if run:
        info_message(' Job started')
    if wait:
        info_message(' Waiting for job to finish')
        try:
            job_status = created_job.wait_for_finish()
        except TimeOutError:
            error_message(f"Timed out while waiting for job {created_job.job_id}. "
                          f"Your job may still be running. "
                          f"Use 'disco job view' to check the job's status")
            return
    display_job(created_job.job_id, job_status)

@job_commands.command('start', context_settings=CONTEXT_SETTINGS, short_help="Start an existing job.")
@click.argument('job_id', metavar='<JOB_ID>')
def start_job(job_id):
    """
    Start an existing job.

    $ disco job start <job_id>

    You may use ‘job list’ command to get the job id.
    """
    if not verify_logged_in():
        return
    try:
        job = Job(job_id)
        job.start()
        success_message(f"Job {job_id} started")
    except DiscoException as ex:
        error_message("Error occurred while starting job")
        exception_message(ex)


@job_commands.command('list', context_settings=CONTEXT_SETTINGS, short_help="List all jobs and their status.")
def list_jobs():
    """
    List all jobs and their status.

    $ disco job list

    Status Decription: \n

    Deleted\t The job failed\n\r
    Done\t The job is done\n
    Failed\t The job failed\n
    Listed\t The job is queued\n
    Stopped\t The job has stopped\n
    Unknown\t The job has no status\n
    Working\t The job is working\n
    """

    if not verify_logged_in():
        return
    try:
        job_list = Job.list_jobs()
        display_job_list(job_list)
    except DiscoException as ex:
        exception_message(ex)


@job_commands.command('stop', context_settings=CONTEXT_SETTINGS, short_help="Stop a running job.")
@click.argument('job_id', metavar='<JOB_ID>')
def stop_job(job_id):
    """
    Stop a running job.

    $ disco job stop <job_id>

    You may use ‘job list’ command to get the job id.
    """

    if not verify_logged_in():
        return
    try:
        job = Job(job_id)
        job.stop()
        success_message(f"Stopping job {job_id}")
    except DiscoException as ex:
        exception_message(ex)


@job_commands.command('archive', context_settings=CONTEXT_SETTINGS, short_help="Archive a specific job.")
@click.argument('job_id', metavar='<JOB_ID>')
def archive_job(job_id):
    """
    Archive a specific job.

    $ disco job archive <job_id>

    You may use ‘job list’ command to get the job id.
    """

    if not verify_logged_in():
        return

    try:
        job = Job(job_id)
        job.archive()
        success_message(f"Job {job_id} was archived")
    except DiscoException as ex:
        error_message("Failed to archive job")
        exception_message(ex)


@job_commands.command('view', context_settings=CONTEXT_SETTINGS, short_help="View specific job's details.")
@click.argument('job_id', metavar='<JOB_ID>')
def view_job(job_id):
    """
    View specific job's details.

    $ disco job view <job_id>

    You may use ‘job list’ command to get the job id.
    """

    if not verify_logged_in():
        return
    try:
        job = Job(job_id)
        details = job.get_details()
        display_job_details(details)
    except DiscoException as ex:
        error_message("Error occurred while getting job details")
        exception_message(ex)


@job_commands.command('download-results', context_settings=CONTEXT_SETTINGS, short_help="Download results of a job.")
@click.argument('job_id', metavar='<JOB_ID>')
@click.option('-d', '--destination', prompt="Path for result files",
              help="The destination path to download the result files to.")
def download_results(job_id, destination):
    """
    Download results of a job.

    $ disco job download-results <job_id> -d <results files path>

    You may use ‘job list’ command to get the job id.
    """

    if not verify_logged_in():
        return
    dest_dir = Path(destination)
    if not dest_dir.exists():
        error_message(f"{destination} doesn't exist")
        return
    if not dest_dir.is_dir():
        error_message("Destination must be a folder")
        return

    try:
        task_results = Job(job_id).get_results(block=True)
        for task_result in task_results:
            task_dir = os.path.join(destination, f"{int(time.time() * 1000)}-{task_result.task_id}")
            safe_make_dirs(task_dir)
            task_result.write_files(task_dir)
            success_message("Results downloaded successfully")
    except DiscoException as ex:
        error_message(f"Error occurred while getting job {job_id} results")
        exception_message(ex)


def display_job_list(job_list):
    """Receives a list of jobs and prints them as a table"""
    tabulated_array = []
    for job in job_list:
        tabulated_array.append([job.id, job.name, job.status])
    displayable_table = list_to_table(tabulated_array, ["ID", "Name", "Status"])
    info_message(displayable_table)


def display_job(job, job_status=None):
    """Displays the job id on the screen"""
    if job_status is None:
        info_message(f"Created job with id {job}")
    elif job_status == JobStatus.failed:
        error_message(f"Job {job} failed")
    elif job_status == JobStatus.done:
        success_message(f"Job {job} finished successfully")
    else:
        info_message(f"Created job with id {job}")


def display_job_details(job: JobDetails):
    """Receives a job details and prints it to the screen"""
    info_message(f"Name: {job.name}")
    info_message(f"Status: {job.status}")
    info_message('Task summary')
    info_message('------------')
    info_message(f"Waiting: {job.tasks_summary.queued}")
    info_message(f"queued: {job.tasks_summary.queued}")
    info_message(f"running: {job.tasks_summary.running}")
    info_message(f"failed: {job.tasks_summary.failed}")
    info_message(f"success: {job.tasks_summary.success}")


def get_file_list(file_list_string):
    """
    Args:
        file_list_string:

    Returns:
        list
    """
    file_paths = []
    split_file_list = file_list_string.split(',')
    for file_path in split_file_list:
        full_file_path = os.path.expanduser(file_path)
        if full_file_path.find('*') >= 0:
            files = glob.glob(full_file_path)
            files.sort(key=os.path.getmtime)
            file_paths += [Path(file) for file in files]
            continue
        path = Path(full_file_path)
        if not path.exists():
            raise FileNotFoundError(f"{full_file_path} doesn't exist")
        if path.is_dir():
            expanded_file = list(Path(full_file_path).glob('**/*'))
            if len(expanded_file) == 0:
                raise FileNotFoundError(f"Folder {full_file_path} is empty")
            file_paths += expanded_file
        else:
            file_paths.append(path)
    return file_paths

def upload_job_script_file(script, cluster_id, show_progress_bar):
    """
    Args:
        script: path to script file.
        cluster_id: cluster id.
        show_progress_bar (bool):

    Returns:
        DB Id for script file.
    """
    script_file_id = upload_file(Path(script).name, Path(script), cluster_id, show_progress_bar)
    info_message(" Script file uploaded")
    return script_file_id

def upload_job_files(input_paths, const_paths, cluster_id, show_progress_bar=True):
    """
    Args:
        input_paths: list of paths to input file
        const_paths: list of paths to constant files
        cluster_id: cluster id
        show_progress_bar (bool):

    Returns:
        DB Ids for constant files and input files

    """
    input_file_ids = []
    for input_file in input_paths:
        input_file_id = upload_file(input_file.name, input_file, cluster_id, show_progress_bar)
        input_file_ids.append(input_file_id)
        info_message(f" Uploaded input file {input_file}")
    const_file_ids = []
    for const_file in const_paths:
        const_file_id = upload_file(const_file.name, const_file, cluster_id, show_progress_bar)
        const_file_ids.append(const_file_id)
        info_message(f" Uploaded constants file {const_file}")
    return const_file_ids, input_file_ids


def _get_bucket_input_file_ids(raw_bucket_paths, cluster_id):
    """
    Args:
        raw_bucket_paths (str):
        cluster_id (str):

    Returns:
        (list, bool): list of file ids and a flag if we got an error
    """

    is_error = True
    empty_result = ([], is_error)

    if not cluster_id:
        error_message("Please specify the cluster ID which contains your bucket, using `--cluster-id`")
        return empty_result

    bucket_paths = raw_bucket_paths.split(',')
    try:
        bucket_input_file_ids = input_files_from_bucket(bucket_paths, cluster_id)
    except BucketPathsException as error:
        invalid_bucket_paths = sorted([bucket_path for bucket_path, error_type in error.bucket_paths_errors.items()
                                       if error_type == BucketPathsErrorTypes.InvalidPath])

        empty_bucket_paths = sorted([bucket_path for bucket_path, error_type in error.bucket_paths_errors.items()
                                     if error_type == BucketPathsErrorTypes.NoFilesInPath])

        if len(invalid_bucket_paths) > 0:
            error_message('Invalid or missing buckets paths specified:')
            for bucket_path in invalid_bucket_paths:
                error_message(bucket_path)

        if len(empty_bucket_paths) > 0:
            error_message('No input files found in the following buckets paths:')
            for bucket_path in empty_bucket_paths:
                error_message(bucket_path)

        return empty_result

    if len(bucket_input_file_ids) == 0:
        error_message('No input files found in the specified buckets paths')
        return empty_result

    info_message(f'Found {len(bucket_input_file_ids)} input files in your bucket')

    return bucket_input_file_ids, False


def safe_make_dirs(dir_path):
    """

    Args:
        dir_path: directory to create

    Returns:

    """
    try:
        os.makedirs(dir_path)
    except OSError as ex:
        if ex.errno != errno.EEXIST:
            raise



import pkg_resources
import requests
import packaging.version

DISCO_PACKAGE = 'disco'


def _installed_version(package):
    """Gets the version of an installed (pip) cli package

    Args:
        package: the package name.

    Returns:
        str: The version of the installed package.
    """
    try:
        dist = pkg_resources.get_distribution(package)
    except pkg_resources.DistributionNotFound:
        return None

    return str(dist.version)


def disco_cli_version():
    """Gets Dis.co's cli version.

    Returns:
        str: The version of the installed package.
    """
    return _installed_version(DISCO_PACKAGE)


def _latest_version(package=None):
    """Gets the version of package on pypi.python.org

    Args:
        package: the package name.

    Returns:
        str: The latest version of the package.
    """
    try:
        package = package or 'disco'
        url = f'https://pypi.python.org/pypi/{package}/json'
        req = requests.get(url)
        if req.status_code == requests.codes.ok:  # pylint: disable=no-member
            data = req.json()
            version = data['info']['version']
            return version
        return None
    except Exception:  # pylint: disable=broad-except
        return None


def is_update_needed():
    """Checks whether there is a newer version available

    Returns
        bool: 'True' if there is a newer version, 'False' otherwise
    """
    current_ver = disco_cli_version()
    latest_ver = _latest_version(DISCO_PACKAGE)

    if current_ver is None or latest_ver is None:
        return False

    return packaging.version.parse(latest_ver) > packaging.version.parse(current_ver)



from disco.base_controller import BaseController
from disco.models import RepositoryDetails


class Repository(BaseController):
    """
    Repository methods
    """

    @classmethod
    def list_repositories(cls, limit=None, next_=None):
        """Show a list of all the repositories of this user.

                Args:
                    limit (int): pagination limit
                    next_: pagination next

                Returns:
                    list(): List of the repositories of this user.
                """

        res = cls.query('getArtifactsRepositories', limit=limit,
                        ownerId=cls.get_current_user_id(), next=next_)

        if res is None:
            return []

        active_repositories = [RepositoryDetails(repository) for repository in res['getArtifactsRepositories']
                               if repository['isActive']]

        return active_repositories



from click.testing import CliRunner
from mock import patch
from requests import ConnectionError, HTTPError

from tests.base_test import BaseTest
from disco_cli import main, setup_cli


class TestCli(BaseTest):

    def setup_class(self):
        setup_cli()

    @patch('disco_cli.cli')
    @patch('click.echo')
    def test_no_connection(self, click_echo, cli):
        cli.side_effect = ConnectionError
        main()
        click_echo.assert_called_with("\x1b[31mCouldn't establish internet connection. "
                                      "Please connect to the internet and try again\x1b[0m")

    @patch('disco_cli.cli')
    @patch('click.echo')
    def test_unknown_exception(self, click_echo, cli):
        cli.side_effect = InterruptedError
        main()
        click_echo.assert_called_with("\x1b[31mUnknown error occurred\x1b[0m")

    @patch('disco_cli.cli')
    @patch('click.echo')
    def test_unknown_server_error(self, click_echo, cli):
        cli.side_effect = HTTPError
        main()
        click_echo.assert_called_with("\x1b[31mUnknown error received from server\x1b[0m")





from tests.base_test import BaseTest
from disco.core.exceptions import RequestException, BucketPathsException, \
    DiscoErrorCodes
from disco.core.exception_factory import ExceptionFactory


class TestUtils(BaseTest):

    def test_from_response_no_message_with_http_status_code(self):
        http_status_code = 404
        json_response = {'code': http_status_code}

        result = ExceptionFactory.from_response(json_response)

        assert isinstance(result, RequestException)
        assert str(result) == f'{RequestException.DEFAULT_MESSAGE} ({json_response.get("code")})'
        assert result.additional_info == {}

    def test_from_response_no_error_code(self):
        error_message = self.random_str('error_message')
        additional_info = {'errorCode': None}
        json_response = {'error': {'message': error_message}, 'additional_info': additional_info}

        result = ExceptionFactory.from_response(json_response)

        assert isinstance(result, RequestException)
        assert str(result) == error_message
        assert result.additional_info == additional_info

    def test_from_response_error_code_bucket_paths(self):
        error_message = self.random_str('error_message')
        bucket_paths_errors = self.random_dict(key_prefix='bucket_paths_errors')
        additional_info = {'errorCode': DiscoErrorCodes.BucketPaths, 'bucketPaths': bucket_paths_errors}
        json_response = {'error': {'message': error_message}, 'additional_info': additional_info}

        result = ExceptionFactory.from_response(json_response)

        assert isinstance(result, BucketPathsException)
        assert str(result) == error_message
        assert result.bucket_paths_errors == bucket_paths_errors



from .base_model import BaseModel


class JobDetails(BaseModel):
    """
    Job Details
    """

    @property
    def id(self):
        """
        Returns:
            str
        """
        return self._data.get('id')

    @property
    def status(self):
        """
        Returns:
            str
        """
        return self._data.get('status')

    @property
    def name(self):
        """
        Returns:
            str
        """
        return self._data.get('request', {}).get('meta', {}).get('name')

    @property
    def tasks_summary(self):
        """
        Returns:
            TasksSummary
        """
        summary_data = self._data.get('tasksSummary')
        if not summary_data:
            return None

        return TasksSummary(**summary_data)


class TasksSummary:  #pylint: disable=redefined-builtin,too-many-instance-attributes
    """
    Tasks Summary
    """

    # pylint: disable=too-many-arguments
    def __init__(self, waiting=0, queued=0, running=0, failed=0, success=0, timeout=0, lost=0, unknown=0, all=0):
        self.queued = queued
        self.running = running
        self.failed = failed
        self.success = success
        self.timeout = timeout
        self.waiting = waiting
        self.lost = lost
        self.unknown = unknown
        self.all = all



import click
from .context_state import CliContextState


def quite_mode_option(func):
    """
    Add quite mode flag to context state

    Args:
        func:
    """
    def callback(ctx, _, value):
        state = ctx.ensure_object(CliContextState)
        state.quite_mode = value
        return value
    return click.option('-q', '--quite', default=False, is_flag=True, required=False, help='Run in quite mode.',
                        expose_value=False, callback=callback)(func)


def common_options(func):
    """
    Adds common options used by multiple CLI commands

    Args:
        func:
    """
    func = quite_mode_option(func)
    return func



#  Copyright (c) 2019 Samsung, Inc. and its affiliates.
#
#  This source code is licensed under the RESTRICTED license found in the
#  LICENSE file in the root directory of this source tree.

import textwrap
import uuid

import pytest

import disco
from disco.core.constants import JobStatus, TaskStatus
from tests.base_test import BaseTest
from tests.live import env
from .env import LIVE_TESTS_TIMEOUT_SECONDS


@pytest.mark.skipif(env.skip, reason=env.reason)
# TODO: Test with a non-default cluster
# TODO: Archive jobs on tear-down even if an exception was thrown
class TestJobsLive(object):

    def setup_class(self):
        BaseTest.disable_progress_bar()

    def test_start_job(self):
        script_content = 'print(\'Hello from automation!\')'
        script_file_id = disco.upload_file('hello.py', script_content)
        input_file_id = disco.upload_file('input.txt', '13')
        constants_file_id = disco.upload_file('constants.txt', 'ZZZ')

        job_name = 'Automation Start Job %s' % uuid.uuid4()
        job = disco.Job.create(script_file_id=script_file_id,
                               input_file_ids=[input_file_id],
                               constants_file_ids=[constants_file_id],
                               job_name=job_name)

        job.start()
        job_details = job.get_details()
        job_id = job_details.id

        assert job_id
        assert job_details.name == job_name

        print(f'Waiting for job {job.job_id} to finish...')

        job.wait_for_finish(interval=10, timeout=LIVE_TESTS_TIMEOUT_SECONDS)
        assert job.get_status() == JobStatus.done

        print(f'job {job.job_id} finished!')

        job_tasks = job.get_tasks()
        assert job_tasks[0].status == TaskStatus.success.value

        jobs_list = disco.Job.list_jobs()
        assert job_id in [job_details.id for job_details in jobs_list]

        job.archive()

    def test_create_job_from_git_repository(self):
        input_file_id = disco.upload_file('input.txt', '13')
        constants_file_id = disco.upload_file('constants.txt', 'ZZZ')
        users_repositories = disco.Repository.list_repositories()
        if len(users_repositories) > 0:
            script_repo_id = users_repositories[0].id;
            script_file_path = 'aaa'
            job_name = 'Automation Start Job %s' % uuid.uuid4()
            job = disco.Job.create(input_file_ids=[input_file_id],
                                   constants_file_ids=[constants_file_id],
                                   job_name=job_name,
                                   script_repo_id = script_repo_id,
                                   script_file_path_in_repo= script_file_path)

            job_details = job.get_details()
            job_id = job_details.id
            assert job_id
            assert job_details.name == job_name

            job.archive()

    def test_cancel_job(self):
        script_content = textwrap.dedent('''
        import time
        
        for i in range(9000):
            print(i)
            time.sleep(1)
        ''')

        script_file_id = disco.upload_file('sleep.py', script_content)

        job_name = 'Automation Cancel Job %s' % uuid.uuid4()
        job = disco.Job.create(script_file_id=script_file_id,
                               job_name=job_name)

        print(f'Starting job {job.job_id}...')

        job.start()
        job_status = job.wait_for_status(JobStatus.working, interval=10,
                                         timeout=LIVE_TESTS_TIMEOUT_SECONDS)
        assert job_status == JobStatus.working

        sleep_time_sec = 10

        print(f'Sleeping for {sleep_time_sec}s ...')  # give the task time to really start running

        import time
        time.sleep(sleep_time_sec)

        print(f'Job {job.job_id} started, stopping job...')

        job.stop()

        print(f'Waiting for job {job.job_id} to finish...')

        # TODO: Distinguish between failed or completed and stopped jobs on the
        #       server side! Currently we can't tell a stopped job from a
        #       completed job that has a failed task
        job_status = job.wait_for_status(JobStatus.done, interval=10,
                                         timeout=LIVE_TESTS_TIMEOUT_SECONDS)
        assert job_status == JobStatus.done

        print(f'job {job.job_id} finished!')

        tasks = job.get_tasks()
        assert len(tasks) == 1
        assert tasks[0].status == TaskStatus.failed.value

        job.archive()

    def test_auto_start_job(self):
        script_content = 'print(\'Hello from automation!\')'
        script_file_id = disco.upload_file('hello.py', script_content)

        job_name = 'Automation Auto Start Job %s' % uuid.uuid4()
        job = disco.Job.create(script_file_id=script_file_id,
                               job_name=job_name, auto_start=True)

        print(f'job {job.job_id} finished!')

        job.wait_for_finish(interval=10, timeout=LIVE_TESTS_TIMEOUT_SECONDS)
        assert job.get_status() == JobStatus.done

        job.archive()

    def test_list_jobs(self):
        jobs = disco.Job.list_jobs(1)
        assert isinstance(jobs, list)
        if jobs:
            assert len(jobs) == 1
            job_details = jobs[0]
            assert job_details.id is not None
            assert job_details.name is not None
            assert job_details.status is not None

    def test_exception_on_job(self):
        erroneous_script = '1 / 0'
        script_file_id = disco.upload_file('woof.py', erroneous_script)

        job_name = 'Automation Failing Job %s' % uuid.uuid4()
        job = disco.Job.create(script_file_id, job_name=job_name, auto_start=True)

        print(f'Waiting for job {job.job_id} to finish...')

        job.wait_for_finish(interval=10, timeout=LIVE_TESTS_TIMEOUT_SECONDS)
        assert job.get_status() == JobStatus.done

        print(f'job {job.job_id} finished!')

        job_tasks = job.get_tasks()
        assert job_tasks[0].status == TaskStatus.failed.value

        job.archive()



from disco.core.exceptions import GraphQLRequestException
from disco.models import JobDetails

MockListJobsResponse = [
    JobDetails({'id': '5df8db9f099035000c7b0c87', 'status': 'Done', 'request': {'meta': {'name': 'dsfgfdsgs'}}}),
    JobDetails({'id': '5df8db7af6f866000c42f04e', 'status': 'Done', 'request': {'meta': {'name': 'asddfgssdfaasd'}}}),
    JobDetails({'id': '5df8d95f32d75f000c3afb8a', 'status': 'Listed', 'request': {'meta': {'name': 'kjhlkjlkhj'}}}),
    JobDetails({'id': '5df8d91032d75f000c3afb81', 'status': 'Listed', 'request': {'meta': {'name': 'jhkjkhjhkkju'}}}),
    JobDetails( {'id': '5df8d86a32d75f000c3afb75', 'status': 'Listed', 'request': {'meta': {'name': 'asffdasdf'}}}),
    JobDetails({'id': '5db18c2fa13a79000f002696', 'status': 'Working', 'request': {'meta': {'name': 'RadioEvent'}}}),
    JobDetails({'id': '5daff5f4e0f21e000a079948', 'status': 'Done','request': {'meta': {'name': 'Dng Territory'}}}),
    JobDetails({'id': '5daea8d0e0f21e000a079397', 'status': 'Done', 'request': {'meta': {'name': 'Caring Territory'}}}),
    JobDetails({'id': '5daea77fe0f21e000a07938f', 'status': 'Done', 'request': {'meta': {'name': 'Absorbing Clover'}}}),
    JobDetails({'id': '5d89ac6493a029000a34a077', 'status': 'Done', 'request': {'meta': {'name': 'Grumpy Teaching'}}}),
    JobDetails({'id': '5d89ac0ffc4015000dd1e94f', 'status': 'Done', 'request': {'meta': {'name': 'Harmonious Boy'}}}),
    JobDetails({'id': '5d88b54afc4015000dd1e3b1', 'status': 'Done', 'request': {'meta': {'name': 'Funny Grape'}}}),
    JobDetails({'id': '5d887bd993a029000a349940', 'status': 'Done', 'request': {'meta': {'name': 'Uplifting Circle'}}}),
    JobDetails({'id': '5d887bac8d9278000a507995', 'status': 'Done', 'request': {'meta': {'name': 'Able Income'}}}),
    JobDetails({'id': '5d66595208edfa000a250dda', 'status': 'Done', 'request': {'meta': {'name': 'Cool Humor'}}})
]

MockViewJobResponse = JobDetails(
    {'id': '5df8db9f099035000c7b0c87', 'request': {'meta': {'name': 'dsfgfdsgs'}}, 'status': 'Done',
     'activities': [{'action': 'Created', 'actor': None, 'timestamp': 1576590239716},
                    {'action': 'Started', 'actor': None, 'timestamp': 1576590239834}],
     'tasksSummary': {'waiting': 0, 'queued': 0, 'running': 0, 'failed': 1, 'success': 0, 'lost': 0,
                      'unknown': 0, 'all': 1}})

MockLoginResponse = {'login': {
    'token': 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VySWQiOiI1ZDY2NTkyMTRmM'
             'jAzMjAwMGI4ZmU4NzYiLCJwcm9maWxlSWQiOiI1ZDY2NTkyMTRmMjAzMjAwMGI4ZmU4'
             'NzkiLCJyb2xlcyI6WyJDb25zb2xlIl0sImlhdCI6MTU3NjY2MzUzNSwiZXhwIjoxNTc'
             '3MjY4MzM1LCJhdWQiOlsiQ29uc29sZSJdLCJzdWIiOiJBdXRoIn0.QFCLt-PMNfzpkg'
             'zNxA2F3FJKClcroECICNclrhoPIrc'}}

MockBadIdException = GraphQLRequestException("", [
    {'message': 'Cast to ObjectId failed for value "5df8db9f099035000c7b0c87afdasdsa" at path "_id" for model "Job"',
     'locations': [{'line': 2, 'column': 13}], 'path': ['fetchJob'], 'extensions': {'code': 'INTERNAL_SERVER_ERROR',
                                                                                    'exception': {'errors': [{
                                                                                        'message': 'Cast to ObjectId '
                                                                                                   'failed for value '
                                                                                                   '"5df8db9f099035000c7b0c87afdasdsa" at path "_id" for model "Job"',
                                                                                        'locations': [],
                                                                                        'path': [
                                                                                            'fetchJob']}]}}}])



import click


class CliContextState:
    """
    CLI Context State
    """

    def __init__(self, quite_mode=False):
        """
        Args:
            quite_mode (bool): Run in quite mode
        """
        self.quite_mode = quite_mode


pass_state = click.make_pass_decorator(CliContextState, ensure=True)  # pylint: disable=invalid-name



from tests.base_test import BaseTest
from disco_cli import cli, setup_cli
from mock import patch
from click.testing import CliRunner
from .cli_test_utils import output_message_includes
from disco.models import RepositoryDetails

class TestRepository(BaseTest):

    def setup_class(self):
        setup_cli()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Repository.list_repositories')
    def test_list_repositories(self, list_repositories_mock, is_logged_in_mock):
        is_logged_in_mock.return_value = True
        list_repositories = [
            RepositoryDetails({'id': '5e143698be608b000c6064b0', 'displayName': 'test repository 1', 'is_active': True}),
            RepositoryDetails({'id': '5e1c62f8a1c673000c1e68a3', 'displayName': 'test repository 2', 'is_active': True})
        ]

        list_repositories_mock.return_value = list_repositories
        runner = CliRunner()
        result = runner.invoke(cli, ['repository', 'list'])
        assert result.exit_code == 0
        list_repositories_mock.assert_called()
        assert output_message_includes(result, '| 5e143698be608b000c6064b0 | test repository 1 |')
        assert output_message_includes(result, '| 5e1c62f8a1c673000c1e68a3 | test repository 2 |')

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Repository.list_repositories')
    def test_empty_list_repositories(self, list_repositories_mock, is_logged_in_mock):
        is_logged_in_mock.return_value = True
        list_repositories = []

        list_repositories_mock.return_value = list_repositories
        runner = CliRunner()
        result = runner.invoke(cli, ['repository', 'list'])
        assert result.exit_code == 0
        list_repositories_mock.assert_called()
        assert result.output == 'There are no set repositories for your account\n'







import pytest

import disco
from tests.live import env


@pytest.mark.skipif(env.skip, reason=env.reason)
class TestClustersLive(object):
    def test_list_jobs(self):
        clusters = disco.Cluster.list_clusters()
        assert isinstance(clusters, list)
        if clusters:
            assert clusters[0].name == 'discoCloud'
            assert clusters[0].is_default
            assert clusters[0].id is not None
            assert clusters[0].type == 'aws'
            assert clusters[0].region is not None
            assert clusters[0].external_account_id is not None
            assert clusters[0].bucket_name is not None
            assert clusters[0].is_last_connection_check_valid
            assert clusters[0].is_active



from pathlib import Path
from time import sleep
import os
from disco import Job
from disco.core.exceptions import BucketPathsException, BucketPathsErrorTypes
from disco.core.constants import JobStatus
from disco.task import TaskResult

from .sdk_mocks import MockListJobsResponse, MockViewJobResponse, MockBadIdException
from mock import patch, call
from click.testing import CliRunner
from disco_cli import cli, setup_cli
from .cli_test_utils import output_message_includes
from tests.base_test import BaseTest


class TestJobCommands(BaseTest):

    def setup_class(self):
        setup_cli()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    def test_job_list(self, is_logged_in_mock):
        """
        Tests job list
        Returns:

        """
        is_logged_in_mock.return_value = True
        with patch('disco.Job.list_jobs') as list_jobs_mock:
            list_jobs_mock.return_value = MockListJobsResponse
            runner = CliRunner()
            result = runner.invoke(cli, ['job', 'list'])
            assert result.exit_code == 0
            list_jobs_mock.assert_called()
            is_logged_in_mock.assert_called()
            assert output_message_includes(result, '| 5d66595208edfa000a250dda | Cool Humor       | Done     |')

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.list_jobs')
    def test_action_without_login(self, list_jobs_mock, is_logged_in_mock):
        list_jobs_mock.return_value = []
        is_logged_in_mock.return_value = False
        runner = CliRunner()
        result = runner.invoke(cli, ['job', 'list'])
        assert result.exit_code == 0
        assert result.output == "You must be logged in to perform this operation\n"

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.get_details')
    def test_view_job(self, get_details_mock, is_logged_in_mock):
        """
        Tests a successful job view command
        Returns:

        """
        is_logged_in_mock.return_value = True
        get_details_mock.return_value = MockViewJobResponse
        runner = CliRunner()
        result = runner.invoke(cli, ['job', 'view', 'job_id'])
        get_details_mock.assert_called_once()
        assert result.exit_code == 0
        assert output_message_includes(result, "Status: Done")
        assert output_message_includes(result, "Name: dsfgfdsgs")
        assert output_message_includes(result, "failed: 1")

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.get_details')
    def test_view_job_malformed_id(self, get_details_mock, is_logged_in_mock):
        is_logged_in_mock.return_value = True
        get_details_mock.side_effect = MockBadIdException
        get_details_mock.return_value = MockViewJobResponse
        runner = CliRunner()
        result = runner.invoke(cli, ['job', 'view', 'job_id'])
        get_details_mock.assert_called_once()
        assert result.exit_code == 0
        assert output_message_includes(result, "Bad format for Id")

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    def test_create_job_input_file(self, asset_upload_mock, path_is_dir_mock,
                                   path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using input files
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py -i input_file".split(' '))
                assert result.exit_code == 0
                assert output_message_includes(result, 'Created job with id job_id')
                job_create_mock.assert_called_with(script_file_id='file_id', input_file_ids=['file_id'],
                                                   constants_file_ids=[], job_name='job_name', cluster_instance_type='s',
                                                   cluster_id=None, script_repo_id=None, script_file_path_in_repo=None,
                                                   auto_start=False, upload_requirements_file=True, docker_image_id=None
                                                   )

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('disco.asset.Asset.upload')
    @patch('disco.asset.Asset.input_files_from_bucket')
    def test_create_job_with_bucket_input_files(self, input_files_from_bucket_mock, asset_upload_mock,
                                                job_create_mock, is_logged_in_mock):
        """
        Create job with input files from bucket paths
        """
        bucket_paths = self.random_list('bucket_path', length=3)
        raw_bucket_paths = ','.join(bucket_paths)
        bucket_input_file_ids = self.random_list('bucket_input_file_id', length=3)

        cluster_id = self.random_str('cluster_id')
        job_name = self.random_str('job_name')
        script_file_id = self.random_str('script_file_id')
        script_file_name = 'script_file.py'
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        job_create_mock.return_value = Job(job_id)
        asset_upload_mock.return_value = script_file_id
        input_files_from_bucket_mock.return_value = bucket_input_file_ids
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open(script_file_name, 'w') as f:
                f.write("print('hi!')")
                command = f"job create " \
                          f" --name {job_name} "\
                          f" --script {script_file_name}"\
                          f" --bucket {raw_bucket_paths}" \
                          f" --cluster-id {cluster_id}"
                result = runner.invoke(cli, command)
                assert result.exit_code == 0
                assert output_message_includes(result, f'Found {len(bucket_input_file_ids)} input files in your bucket')
                assert output_message_includes(result, f'Created job with id {job_id}')

                input_file_ids = bucket_input_file_ids

                input_files_from_bucket_mock.assert_called_with(bucket_paths, cluster_id)

                job_create_mock.assert_called_with(script_file_id=script_file_id, input_file_ids=input_file_ids,
                                                   constants_file_ids=[],
                                                   job_name=job_name, docker_image_id=None, script_repo_id=None,
                                                   script_file_path_in_repo=None,
                                                   cluster_instance_type='s', cluster_id=cluster_id, auto_start=False,
                                                   upload_requirements_file=True)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('disco.asset.Asset.upload')
    @patch('disco.asset.Asset.input_files_from_bucket')
    def test_create_job_with_bucket_input_files_no_files_found(self, input_files_from_bucket_mock, asset_upload_mock,
                                                               job_create_mock, is_logged_in_mock):
        """
        Create job with input files from bucket paths - no files found
        """
        bucket_paths = self.random_list('bucket_path', length=3)
        raw_bucket_paths = ','.join(bucket_paths)
        no_bucket_input_file_ids = []

        cluster_id = self.random_str('cluster_id')
        job_name = self.random_str('job_name')
        script_file_id =  self.random_str('script_file_id')
        script_file_name = 'script_file.py'
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        job_create_mock.return_value = Job(job_id)
        asset_upload_mock.return_value = script_file_id
        input_files_from_bucket_mock.return_value = no_bucket_input_file_ids
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open(script_file_name, 'w') as f:
                f.write("print('hi!')")
                command = f"job create " \
                          f" --name {job_name} "\
                          f" --script {script_file_name}"\
                          f" --bucket {raw_bucket_paths}" \
                          f" --cluster-id {cluster_id}"
                result = runner.invoke(cli, command)
                assert result.exit_code == 0
                assert result.output == 'No input files found in the specified buckets paths\n'

                input_files_from_bucket_mock.assert_called_with(bucket_paths, cluster_id)

                job_create_mock.assert_not_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('disco.asset.Asset.upload')
    @patch('disco.asset.Asset.input_files_from_bucket')
    def test_create_job_with_bucket_input_files_error_bucket_path(self, input_files_from_bucket_mock, asset_upload_mock,
                                                                  job_create_mock, is_logged_in_mock):
        """
        Create job with input files from bucket paths - no files found
        """
        valid_bucket_paths = self.random_list('valid_bucket_paths', length=2)
        invalid_bucket_paths = self.random_list('invalid_bucket_paths', length=2)
        empty_bucket_paths = self.random_list('empty_bucket_paths', length=2)
        bucket_paths = valid_bucket_paths + invalid_bucket_paths + empty_bucket_paths
        raw_bucket_paths = ','.join(bucket_paths)

        bucket_paths_errors = {
            invalid_bucket_paths[0]: BucketPathsErrorTypes.InvalidPath,
            invalid_bucket_paths[1]: BucketPathsErrorTypes.InvalidPath,
            empty_bucket_paths[0]: BucketPathsErrorTypes.NoFilesInPath,
            empty_bucket_paths[1]: BucketPathsErrorTypes.NoFilesInPath,
        }

        cluster_id = self.random_str('cluster_id')
        job_name = self.random_str('job_name')
        script_file_id =  self.random_str('script_file_id')
        script_file_name = 'script_file.py'
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        job_create_mock.return_value = Job(job_id)
        asset_upload_mock.return_value = script_file_id
        input_files_from_bucket_mock.side_effect = BucketPathsException('some error message',
                                                                        bucket_paths_errors=bucket_paths_errors)
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open(script_file_name, 'w') as f:
                f.write("print('hi!')")
                command = f"job create " \
                          f" --name {job_name} "\
                          f" --script {script_file_name}"\
                          f" --bucket {raw_bucket_paths}" \
                          f" --cluster-id {cluster_id}"
                result = runner.invoke(cli, command)
                assert result.exit_code == 0

                sorted_list_invalid_bucket_paths_output = '\n'.join(sorted(invalid_bucket_paths))
                sorted_list_empty_bucket_paths_output = '\n'.join(sorted(empty_bucket_paths))

                expected_result_output = f'Invalid or missing buckets paths specified:\n' \
                                         f'{sorted_list_invalid_bucket_paths_output}\n' \
                                         f'No input files found in the following buckets paths:\n' \
                                         f'{sorted_list_empty_bucket_paths_output}\n'

                assert result.output == expected_result_output

                input_files_from_bucket_mock.assert_called_with(bucket_paths, cluster_id)

                job_create_mock.assert_not_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('disco.asset.Asset.upload')
    @patch('disco.asset.Asset.input_files_from_bucket')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    def test_create_job_with_bucket_and_local_input_files(self, path_is_dir_mock, path_exists_mock,
                                                          input_files_from_bucket_mock, asset_upload_mock,
                                                          job_create_mock, is_logged_in_mock):
        """
        Create job with input files from bucket paths and local input paths
        """
        bucket_paths = self.random_list('bucket_path', length=3)
        raw_bucket_paths = ','.join(bucket_paths)
        bucket_input_file_ids = self.random_list('bucket_input_file_id', length=3)

        input_paths = self.random_list('input_paths', length=3)
        raw_input_paths = ','.join(input_paths)
        local_input_file_ids = self.random_list('local_input_file_id', length=3)

        cluster_id = self.random_str('cluster_id')
        job_name = self.random_str('job_name')
        script_file_id = self.random_str('script_file_id')
        script_file_name = 'script_file.py'
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job(job_id)
        asset_upload_mock.side_effect = local_input_file_ids + [script_file_id]
        input_files_from_bucket_mock.return_value = bucket_input_file_ids
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open(script_file_name, 'w') as f:
                f.write("print('hi!')")
                command = f"job create " \
                          f" --name {job_name} "\
                          f" --script {script_file_name}" \
                          f" --input {raw_input_paths}" \
                          f" --bucket {raw_bucket_paths}" \
                          f" --cluster-id {cluster_id}"
                result = runner.invoke(cli, command)
                assert result.exit_code == 0
                assert output_message_includes(result, f'Found {len(bucket_input_file_ids)} input files in your bucket')
                assert output_message_includes(result, f'Created job with id {job_id}')

                input_file_ids = bucket_input_file_ids + local_input_file_ids

                input_files_from_bucket_mock.assert_called_with(bucket_paths, cluster_id)

                job_create_mock.assert_called_with(script_file_id=script_file_id, input_file_ids=input_file_ids,
                                                   constants_file_ids=[],
                                                   job_name=job_name, docker_image_id=None, script_repo_id=None,
                                                   script_file_path_in_repo=None,
                                                   cluster_instance_type='s', cluster_id=cluster_id, auto_start=False,
                                                   upload_requirements_file=True)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('disco.asset.Asset.upload')
    def test_create_job_quite_mode(self, asset_upload_mock, job_create_mock, is_logged_in_mock):
        """
        Run create job in quite mode
        """
        is_logged_in_mock.return_value = True
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py --quite")
                assert result.exit_code == 0
                assert output_message_includes(result, 'Created job with id job_id')
                job_create_mock.assert_called_with(script_file_id='file_id', input_file_ids=[], constants_file_ids=[],
                                                   job_name='job_name', cluster_instance_type='s', cluster_id=None,
                                                   script_repo_id=None, script_file_path_in_repo=None, auto_start=False,
                                                   upload_requirements_file=True, docker_image_id=None)

                asset_upload_mock.assert_called_with('script_file.py', Path('script_file.py'),
                                                     cluster_id=None, show_progress_bar=False)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('disco.asset.Asset.upload')
    def test_create_job_quite_mode_disabled_by_default(self, asset_upload_mock, job_create_mock, is_logged_in_mock):
        """
        Run create job, quite mode disabled by default
        """
        is_logged_in_mock.return_value = True
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py")
                assert result.exit_code == 0
                assert output_message_includes(result, 'Created job with id job_id')
                job_create_mock.assert_called_with(script_file_id='file_id', input_file_ids=[], constants_file_ids=[],
                                                   job_name='job_name', cluster_instance_type='s', cluster_id=None,
                                                   script_repo_id=None, script_file_path_in_repo=None, auto_start=False,
                                                   upload_requirements_file=True, docker_image_id=None)

                asset_upload_mock.assert_called_with('script_file.py', Path('script_file.py'),
                                                     cluster_id=None, show_progress_bar=True)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    def test_create_job_without_req_file(self, asset_upload_mock, path_is_dir_mock,
                                         path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using input files
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py -i "
                                            "input_file --dont-generate-req-file".split(' '))
                assert result.exit_code == 0
                assert output_message_includes(result, 'Created job with id job_id')
                job_create_mock.assert_called_with(script_file_id='file_id', input_file_ids=['file_id'],
                                                   constants_file_ids=[], job_name='job_name',
                                                   cluster_instance_type='s', cluster_id=None, script_repo_id=None,
                                                   script_file_path_in_repo=None, auto_start=False,
                                                   upload_requirements_file=False, docker_image_id=None)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    def test_create_job_constant_file(self, asset_upload_mock, path_is_dir_mock,
                                      path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using constant files
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py -c const_file".split(' '))
                assert result.exit_code == 0
                assert output_message_includes(result, 'Created job with id job_id')
                job_create_mock.assert_called_with(script_file_id='file_id', input_file_ids=[],
                                                   constants_file_ids=['file_id'], job_name='job_name',
                                                   cluster_instance_type='s', cluster_id=None, script_repo_id=None,
                                                   script_file_path_in_repo=None, auto_start=False,
                                                   upload_requirements_file=True, docker_image_id=None)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    def test_create_job_many_files(self, asset_upload_mock, path_is_dir_mock,
                                   path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using many input and constant files
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py "
                                            "-i input_file1,input_file2 "
                                            "-c const_file1,const_file2".split(' '))
                assert result.exit_code == 0
                assert output_message_includes(result, 'Created job with id job_id')
                job_create_mock.assert_called_with(script_file_id='file_id', input_file_ids=['file_id', 'file_id'],
                                                   constants_file_ids=['file_id', 'file_id'], job_name='job_name',
                                                   cluster_instance_type='s', cluster_id=None, script_repo_id=None,
                                                   script_file_path_in_repo=None, auto_start=False,
                                                   upload_requirements_file=True, docker_image_id=None)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('pathlib.Path.glob')
    @patch('disco.asset.Asset.upload')
    def test_create_job_directory(self, asset_upload_mock, path_glob_mock, path_is_dir_mock,
                                  path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using directory as input
        Args:
            asset_upload_file_mock:
            path_glob_mock:
            path_is_dir_mock:
            path_exists_mock:
            job_create_mock:

        Returns:

        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = True
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        path_glob_mock.return_value = [Path('file1'), Path('file2')]
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py "
                                            "-i dir_path ")
                assert result.exit_code == 0
                assert output_message_includes(result, 'Created job with id job_id')
                job_create_mock.assert_called_with(script_file_id='file_id', input_file_ids=['file_id', 'file_id'],
                                                   constants_file_ids=[], job_name='job_name',
                                                   cluster_instance_type='s', cluster_id=None, script_repo_id=None,
                                                   script_file_path_in_repo=None, auto_start= False,
                                                   upload_requirements_file=True, docker_image_id=None)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('pathlib.Path.glob')
    @patch('disco.asset.Asset.upload')
    def test_create_job_wildcard(self, asset_upload_mock, path_glob_mock, path_is_dir_mock,
                                 path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using file with wildcards
        """
        script_file_id = self.random_str("script_file_id")
        input_file_id1 = self.random_str("input_file_id1")
        input_file_id2 = self.random_str("input_file_id2")

        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = True
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.side_effect = [input_file_id1, input_file_id2, script_file_id]
        path_glob_mock.return_value = [Path('file1'), Path('file2')]
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f, open("input_file1.txt", 'w') as i1:
                sleep(1)
                i2 = open("input_file2.txt", 'w')
                input_file_contents1 = self.random_str("input_contents1")
                input_file_contents2 = self.random_str("input_contents2")
                i1.write(input_file_contents1)

                i2.write(input_file_contents2)
                i2.close()
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py "
                                            "-i inp* ")
                assert result.exit_code == 0
                assert output_message_includes(result, 'Created job with id job_id')
                job_create_mock.assert_called_with(script_file_id=script_file_id,
                                                   input_file_ids=[input_file_id1, input_file_id2],
                                                   constants_file_ids=[], job_name='job_name',
                                                   cluster_instance_type='s', cluster_id=None, script_repo_id=None,
                                                   script_file_path_in_repo=None, auto_start=False,
                                                   upload_requirements_file=True, docker_image_id=None)

                asset_upload_mock.assert_has_calls([
                    call('script_file.py', Path('script_file.py'), cluster_id=None, show_progress_bar=True),
                    call('input_file1.txt', Path('input_file1.txt'), cluster_id=None, show_progress_bar=True),
                    call('input_file2.txt', Path('input_file2.txt'), cluster_id=None, show_progress_bar=True),
                ], any_order=True)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    def test_create_job_not_supported_script(self, path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Unsuccessful path for creating a job with a script file that is not supported
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file")
                assert result.exit_code == 0
                assert result.output == 'Cannot use script file. ' \
                                        'Currently only Python and bash scripts are supported\n'
                job_create_mock.assert_not_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('pathlib.Path.glob')
    def test_create_job_empty_directory(self, path_glob_mock, path_is_dir_mock,
                                        path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using directory as input
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = True
        path_glob_mock.return_value = []
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py "
                                            "-i dir_path ")
                assert result.exit_code == 0
                assert output_message_includes(result, 'Folder dir_path is empty')
                job_create_mock.assert_not_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    def test_create_job_missing_input(self, path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Unsuccessful path for creating a job with a script file that is not supported
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = False
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli, "job create -n job_name -s script_file.py -i input_file")
                assert result.exit_code == 0
                assert output_message_includes(result, 'input_file doesn\'t exist')
                job_create_mock.assert_not_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    @patch('disco.Job.wait_for_finish')
    def test_create_job_wait_success(self, wait_for_finish_mock, asset_upload_mock, path_is_dir_mock,
                                   path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using input files
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        wait_for_finish_mock.return_value = JobStatus.done
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli,
                                       "job create -n job_name -s script_file.py --run --wait".split(' '))
                assert result.exit_code == 0
                assert output_message_includes(result, 'Job job_id finished successfully')

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    @patch('disco.Job.wait_for_finish')
    def test_create_job_wait_failure(self, wait_for_finish_mock, asset_upload_mock, path_is_dir_mock,
                                     path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Successful path for creating a job using input files
        """
        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.return_value = 'file_id'
        wait_for_finish_mock.return_value = JobStatus.failed
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('script_file.py', 'w') as f:
                f.write("print('hi!')")
                result = runner.invoke(cli,
                                       "job create -n job_name -s script_file.py --run --wait".split(' '))
                assert result.exit_code == 0
                assert output_message_includes(result, 'Job job_id failed')

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('disco.asset.Asset.upload')
    def test_create_job_script_file_in_home_dir(self, asset_upload_mock, job_create_mock, is_logged_in_mock):
        """
        Create job with script file from user's home directory, i.e `~/script_file.py`
        """
        script_file_id = self.random_str("script_file_id")
        input_file_id1 = self.random_str("input_file_id1")
        input_file_id2 = self.random_str("input_file_id2")
        constant_file_id = self.random_str("constant_file_id")

        is_logged_in_mock.return_value = True
        job_create_mock.return_value = Job("job_id")
        asset_upload_mock.side_effect = [input_file_id1, input_file_id2, constant_file_id, script_file_id]
        runner = CliRunner()

        script_filename = 'script_file.py'
        input_filename1 = 'input1.txt'
        input_filename2 = 'input2.txt'
        constant_filename = 'constant.txt'

        script_file_path = os.path.join("~", script_filename)
        input_file_path1 = os.path.join("~", input_filename1)
        input_file_path2 = os.path.join("~", input_filename2)
        constant_file_path = os.path.join("~", constant_filename)

        script_full_file_path = str(Path.home() / script_filename)
        input_file_full_path1 = str(Path.home() / input_filename1)
        input_file_full_path2 = str(Path.home() / input_filename2)
        constant_file_full_path = str(Path.home() / constant_filename)

        try:
            with open(script_full_file_path, 'w') as script_file:
                script_file.write("print('hi!')")

            with open(input_file_full_path1, 'w') as input_file1:
                input_file1.write(self.random_str('input1_content'))

            with open(input_file_full_path2, 'w') as input_file2:
                input_file2.write(self.random_str('input2_content'))

            with open(constant_file_full_path, 'w') as constant_file:
                constant_file.write(self.random_str('constant_file_content'))

            result = runner.invoke(cli, f"job create -n job_name --script {script_file_path} "
                                        f"--input {input_file_path1},{input_file_path2} "
                                        f"--constants {constant_file_path}")
            assert result.exit_code == 0
            assert output_message_includes(result, 'Created job with id job_id')
            job_create_mock.assert_called_with(script_file_id=script_file_id,
                                               input_file_ids=[input_file_id1, input_file_id2],
                                               constants_file_ids=[constant_file_id], job_name='job_name',
                                               cluster_instance_type='s', cluster_id=None, script_repo_id=None,
                                               script_file_path_in_repo=None, auto_start=False,
                                               upload_requirements_file=True, docker_image_id=None)

            asset_upload_mock.assert_has_calls([
                call(script_filename, Path(script_full_file_path), cluster_id=None, show_progress_bar=True),
                call(input_filename1, Path(input_file_full_path1), cluster_id=None, show_progress_bar=True),
                call(input_filename2, Path(input_file_full_path2), cluster_id=None, show_progress_bar=True),
                call(constant_filename, Path(constant_file_full_path), cluster_id=None, show_progress_bar=True),
            ], any_order=True)

        finally:
            os.remove(script_full_file_path)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    def test_create_job_script_file_from_git_repository(self, asset_upload_mock, path_is_dir_mock,
                                    path_exists_mock, job_create_mock, is_logged_in_mock):
        """ Create job with script file from git repository with repository id and path. """
        job_name = self.random_str('job_name')
        input_file_id = self.random_str('input_file_id')
        repository_id = self.random_str('repository_id')
        file_path = 'file_path.py'
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job(job_id)
        asset_upload_mock.return_value = input_file_id
        runner = CliRunner()
        with runner.isolated_filesystem():
            command = f"job create -n {job_name} " \
                      f"-i {input_file_id} " \
                      f"-rep {repository_id} " \
                      f"-f {file_path}"
            result = runner.invoke(cli, command)
            assert result.exit_code == 0
            assert output_message_includes(result, f'Created job with id {job_id}')
            job_create_mock.assert_called_with(script_file_id=None, input_file_ids=[input_file_id],
                                               constants_file_ids=[], job_name=job_name, cluster_instance_type='s',
                                               cluster_id=None, script_repo_id=repository_id,
                                               script_file_path_in_repo=file_path, auto_start=False,
                                               upload_requirements_file=True, docker_image_id=None)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    def test_create_job_script_file_from_git_repository_missing_file_path(self, asset_upload_mock, path_is_dir_mock,
                                    path_exists_mock, job_create_mock, is_logged_in_mock):
        """ Create job with script file from git repository - missing file path. """
        job_name = self.random_str('job_name')
        input_file_id = self.random_str('input_file_id')
        repository_id = self.random_str('repository_id')
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job(job_id)
        asset_upload_mock.return_value = input_file_id
        runner = CliRunner()
        with runner.isolated_filesystem():
            command = f"job create -n {job_name} " \
                      f"-i {input_file_id} " \
                      f"-rep {repository_id} "
            result = runner.invoke(cli, command)
            assert result.exit_code == 0
            assert result.output == "Please specify the script file path using `--file-path`\n"
            job_create_mock.assert_not_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    def test_create_job_script_file_from_git_repository_not_supported_script(self,
                                                                             path_exists_mock, job_create_mock, is_logged_in_mock):
        """
        Unsuccessful path for creating a job with a script file that is not supported
        """
        job_name = self.random_str('job_name')
        input_file_id = self.random_str('input_file_id')
        repository_id = self.random_str('repository_id')
        file_path = self.random_str('file_path')
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        job_create_mock.return_value = Job(job_id)
        runner = CliRunner()
        with runner.isolated_filesystem():
            command = f"job create -n {job_name} " \
                      f"-i {input_file_id} " \
                      f"-rep {repository_id} " \
                      f"-f {file_path}"
            result = runner.invoke(cli, command)
            assert result.exit_code == 0
            assert result.output == 'Cannot use script file. ' \
                                    'Currently only Python and bash scripts are supported\n'
            job_create_mock.assert_not_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    def test_create_job_missing_script_file(self, asset_upload_mock, path_is_dir_mock,
                                    path_exists_mock, job_create_mock, is_logged_in_mock):
        """ Create job - no script file. """
        job_name = self.random_str('job_name')
        input_file_id = self.random_str('input_file_id')
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job(job_id)
        asset_upload_mock.return_value = input_file_id
        runner = CliRunner()
        with runner.isolated_filesystem():
            command = f"job create -n {job_name} -i {input_file_id} "
            result = runner.invoke(cli, command)
            assert result.exit_code == 0
            assert result.output == 'Please specify the script file using either `--script` or `--repository-id` with `--file-path`\n'
            job_create_mock.assert_not_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.create')
    @patch('pathlib.Path.exists')
    @patch('pathlib.Path.is_dir')
    @patch('disco.asset.Asset.upload')
    def test_create_job_script_file_with_docker_image(self, asset_upload_mock, path_is_dir_mock,
                                    path_exists_mock, job_create_mock, is_logged_in_mock):
        """ Create job with script file from git repository with repository id and path. """
        job_name = self.random_str('job_name')
        input_file_id = self.random_str('input_file_id')
        repository_id = self.random_str('repository_id')
        docker_image_id = self.random_str(('docker_img_id'))
        file_path = 'file_path.py'
        job_id = self.random_str('job_id')

        is_logged_in_mock.return_value = True
        path_exists_mock.return_value = True
        path_is_dir_mock.return_value = False
        job_create_mock.return_value = Job(job_id)
        asset_upload_mock.return_value = input_file_id
        runner = CliRunner()
        with runner.isolated_filesystem():
            command = f"job create -n {job_name} " \
                      f"-i {input_file_id} " \
                      f"-rep {repository_id} " \
                      f"-f {file_path} " \
                      f"-did {docker_image_id}"
            result = runner.invoke(cli, command)
            assert result.exit_code == 0
            assert output_message_includes(result, f'Created job with id {job_id}')
            job_create_mock.assert_called_with(script_file_id=None, input_file_ids=[input_file_id],
                                               constants_file_ids=[], job_name=job_name, cluster_instance_type='s',
                                               cluster_id=None, script_repo_id=repository_id,
                                               script_file_path_in_repo=file_path, auto_start=False,
                                               upload_requirements_file=True, docker_image_id=docker_image_id)

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.start')
    def test_start_job(self, job_start_mock, is_logged_in_mock):
        is_logged_in_mock.return_value = True
        runner = CliRunner()
        result = runner.invoke(cli, 'job start job_id')
        assert result.exit_code == 0
        assert result.output == 'Job job_id started\n'
        job_start_mock.assert_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.stop')
    def test_stop_job(self, job_stop_mock, is_logged_in_mock):
        is_logged_in_mock.return_value = True
        runner = CliRunner()
        result = runner.invoke(cli, 'job stop job_id')
        assert result.exit_code == 0
        assert result.output == 'Stopping job job_id\n'
        job_stop_mock.assert_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.archive')
    def test_archive_job(self, job_archive_mock, is_logged_in_mock):
        is_logged_in_mock.return_value = True
        runner = CliRunner()
        result = runner.invoke(cli, 'job archive job_id')
        assert result.exit_code == 0
        assert result.output == 'Job job_id was archived\n'
        job_archive_mock.assert_called()

    @patch('disco.gql.authentication.Authentication.is_logged_in')
    @patch('disco.Job.get_results')
    @patch('pathlib.Path.exists', return_value=True)
    @patch('pathlib.Path.is_dir', return_value=True)
    @patch('os.makedirs')
    @patch('disco.task.TaskResult.write_files')
    def test_download_results(self, write_files_mock, makedirs_mock, is_dir_mock, path_exists_mock,
                              get_results_mock, is_logged_in_mock):
        get_results_mock.return_value = [TaskResult('task_id', {'Iqoqo.stdout.log': 'some output'})]
        is_logged_in_mock.return_value = True
        runner = CliRunner()
        result = runner.invoke(cli, 'job download-results job_id -d destdir')
        assert result.exit_code == 0
        assert result.output == 'Results downloaded successfully\n'
        get_results_mock.assert_called()
        makedirs_mock.assert_called()



import os
import subprocess
import pytest
from disco.core import constants as const
from tests.live import env


@pytest.mark.skipif(env.skip, reason=env.reason)
class TestLiveCLI(object):

    def test_login_list_jobs_logout(self):
        output = subprocess.check_output(['disco', 'login', '-u',
                                          os.environ.get(const.ENV_VAR_EMAIL_NAME), '-p',
                                          os.environ.get(const.ENV_VAR_PASSWORD_NAME)]).decode("utf-8")
        print(output)
        assert output.splitlines()[-1] == "Signed in successfully"
        output = subprocess.check_output(['disco', 'job', 'list']).decode("utf-8")
        assert output.find("Done ") >= 0



# -*- coding: utf-8 -*-
#    ____  __  ____   ___  __
#   (    \(  )/ ___) / __)/  \
#    ) D ( )( \___ \( (__(  O )
#   (____/(__)(____/ \___)\__/

"""
DISCO SDK Library
"""

from .job import Job
from .cluster import Cluster
from .asset import upload_file, input_files_from_bucket, Asset
from .core.credentials import set_credentials
from .repository import Repository
from .docker_image import DockerImage



from requests import ConnectionError as RequestsConnectionError, HTTPError
import click
from disco.core.exceptions import NoCredentials
import cli.utilities.utils as utils
from cli.command_utils import error_message
from cli import auth_commands, cluster_commands, job_commands, version_commands, repository_commands, \
    docker_image_commands

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    """
    Root CLI Command
    """


def setup_cli():
    """
    Setup CLI
    """
    cli.add_command(auth_commands.login)
    cli.add_command(auth_commands.logout)
    cli.add_command(cluster_commands.cluster_commands)
    cli.add_command(job_commands.job_commands)
    cli.add_command(version_commands.cli_version)
    cli.add_command(repository_commands.repository)
    cli.add_command(docker_image_commands.docker)

def check_version():
    """
    Check if there's a newer version of the CLI
    """
    if utils.is_update_needed():
        click.echo(click.style(
            "There is a newer version. \nPlease upgrade using: pip install disco --upgrade\n",
            fg='yellow', bold='True'))


def main():
    """
    Main CLI
    """
    check_version()
    try:
        setup_cli()
    except Exception:  # pylint: disable=broad-except
        print("Error while setting up cli")

    try:
        cli()
    except HTTPError:
        error_message("Unknown error received from server")
    except RequestsConnectionError:
        error_message("Couldn't establish internet connection. "
                      "Please connect to the internet and try again")
    except NoCredentials:
        error_message("You must be logged in to perform this operation")
    except Exception:  # pylint: disable=broad-except
        error_message("Unknown error occurred")


if __name__ == '__main__':
    main()



from .exceptions import DiscoErrorCodes, RequestException, BucketPathsException


class ExceptionFactory:
    """
    Exception Factory
    """

    @classmethod
    def from_response(cls, json_response):
        """
        Args:
            json_response (dict):

        Returns:
            RequestException
        """
        http_status_code = json_response.get('code')
        message = json_response.get('error', {}).get('message') \
            or f'{RequestException.DEFAULT_MESSAGE} ({http_status_code})'

        additional_info = json_response.get('additional_info', {})

        error_code = additional_info.get('errorCode')

        if error_code == DiscoErrorCodes.BucketPaths:
            return BucketPathsException(message, additional_info.get('bucketPaths'))

        return RequestException(message, additional_info)



#  Copyright (c) 2019 Samsung, Inc. and its affiliates.
#
#  This source code is licensed under the RESTRICTED license found in the
#  LICENSE file in the root directory of this source tree.

import os

import mock

from disco.core import constants as const
from .helper import add


class MockTaskResult:
    raw_result_mock = (('result.pickle', b'\x80\x03K\x03.'),
                       ('IqoqoTask.stdout.0.txt',
                        b'result is:  3\nand the result is (drumroll):  3\nresult is:  3\n'),
                       ('IqoqoTask.stderr.0.txt', b''))

    def __init__(self):
        pass

    def get_raw_result(self):
        return self.raw_result_mock


class MockFailedTaskResult:
    raw_result_mock = (('IqoqoTask.stdout.0.txt', b'fail'),
                       ('IqoqoTask.stderr.0.txt', b'error: failed'))

    def __init__(self):
        pass

    def get_raw_result(self):
        return self.raw_result_mock


class MockJob:
    def __init__(self):
        pass

    def start(self):
        pass

    @staticmethod
    def get_results(block):
        return [MockTaskResult()]


class MockFailedJob:
    def __init__(self):
        pass

    def start(self):
        pass

    @staticmethod
    def get_results(block):
        return [MockFailedTaskResult()]


job = MockJob()


@mock.patch.dict(os.environ,
                 {const.ENV_VAR_EMAIL_NAME: 'i@dis.co',
                  const.ENV_VAR_PASSWORD_NAME: 'wheel'},
                 clear=True)
@mock.patch('disco.asset.upload_file', return_value="some_id")
@mock.patch('disco.Job.create', return_value=job)
def test_disco_job_success(job_add_mock, upload_file_mock):
    result = add(1, 2)
    assert upload_file_mock.call_count == 2
    assert job_add_mock.call_count == 1
    # We have two versions here depending on the python version
    assert upload_file_mock.call_args_list[1][0][1] == b'(L1L\nL2L\ntp0\n.' or \
           upload_file_mock.call_args_list[1][0][1] == b'(I1\nI2\ntp0\n.'
    assert job_add_mock.call_args_list[0][0][0] == "some_id"
    assert job_add_mock.call_args_list[0][0][1] == ["some_id"]
    assert result == 3


failed_job = MockFailedJob()


@mock.patch.dict(os.environ,
                 {const.ENV_VAR_EMAIL_NAME: 'i@dis.co',
                  const.ENV_VAR_PASSWORD_NAME: 'wheel'},
                 clear=True)
@mock.patch('disco.asset.upload_file', mock.MagicMock(return_value="some_id"))
@mock.patch('disco.Job.create', mock.MagicMock(return_value=failed_job))
def test_disco_job_fail():
    result = add(1, 2)
    assert not result



from .base_model import BaseModel


class Task(BaseModel):
    """
    Task
    """

    @property
    def id(self):
        """
        Returns:
            str
        """
        return self._data.get('id')

    @property
    def status(self):
        """
        Returns:
            str
        """
        return self._data.get('status')

    @property
    def duration(self):
        """
        Returns:
            int
        """
        return self._data.get('stats', {}).get('duration', 0)

    @property
    def input_file_id(self):
        """
        Returns:
            str
        """
        return self._data.get('request', {}).get('inputFile', {}).get('id')

    @property
    def artifact_ids(self):
        """
        Returns:
            list(str)
        """
        return self._data.get('result', {}).get('artifactIds', [])



"""
Mixins used for backend requests.
The mixins aim to reduce the complexity of the requests classes rather than
creating reusable classes
"""
import json
import logging
import os

import requests

from ..core import constants, utils


class _RequestsMixin:
    """`_RequestsMixin` handles urls generation, error handling and
    common headers for requests to DISCO backend.
    """
    MAX_500_RETRIES = 3
    # 504 is skipped as it it usually caused by long/big requests
    HANDLED_500_CODES = {500, 502}

    @staticmethod
    def handle_500_errors(src_function):
        """Wraps function and retries on 500 errors.
        Args:
            src_function: Function to be executed and retried.

        Returns:
            Wrapped function.
        """

        def _try(*args, **kwargs):
            last_error = None
            for _ in range(_RequestsMixin.MAX_500_RETRIES):
                try:
                    return src_function(*args, **kwargs)
                except requests.HTTPError as http_error:

                    status = http_error.response.status_code
                    if status in _RequestsMixin.HANDLED_500_CODES:
                        logging.warning("Got %s error during request to %s",
                                        status, http_error.response.url)
                        last_error = http_error
                    else:
                        raise
            raise last_error

        return _try

    @classmethod
    def _raw_base_url(cls):
        #Retrieves the base url from various data sources (env, file, default)
        #return: The base url **configured** for DISCO requests
        environ_base_url = os.environ.get(constants.ENV_VAR_BASE_URL_NAME)
        if environ_base_url is not None:
            return environ_base_url

        if constants.DISCO_CONFIG_PATH.exists():
            conf_file = json.loads(
                constants.DISCO_CONFIG_PATH.read_text())
            return conf_file.get('base_url', constants.DEFAULT_BASE_URL)

        return constants.DEFAULT_BASE_URL

    @classmethod
    def base_url(cls):
        """Provides the base url to be used for requests made to
        DISCOs backend.

        Returns:
            str: Base url for DISCO backend server.
        """
        b_url = cls._raw_base_url()
        # allow the old suffix in configuration for backward compatibility
        b_url = utils.str_without_suffix(b_url, '/')
        b_url = utils.str_without_suffix(b_url, constants.REST_API_SUFFIX)
        return b_url

    @classmethod
    def get_graphql_endpoint(cls):
        """Url url for DISCO GQL requests.

        Returns:
            str: Base url for DISCO GQL requests.
        """
        return '/'.join([cls.base_url(), 'graphql'])

    @classmethod
    def get_rest_endpoint(cls, path):
        """Builds url for REST request to the backend servers.

        Returns:
            str: Url for specific REST url.
        """
        rest_part = utils.str_without_prefix(constants.REST_API_SUFFIX)
        path_part = utils.str_without_prefix(path, '/')
        return '/'.join([cls.base_url(), rest_part, path_part])

    @property
    def disco_headers(self):
        """Default headers for REST/GQL request.

        Returns:
            dict: Dictionary of headers.
        """
        return {
            'User-Agent': f"disco-python-sdk version={utils.disco_version()}" \
                f" request_id={self.request_id}"

        }



from tests.base_test import BaseTest
from mock import patch
import mock
import disco


class TestRepositories(BaseTest):

    @BaseTest.with_config_and_env(authenticated=True)
    @patch('disco.gql.authentication.Authentication.get_current_user_id')
    def test_list_repositories(self, get_current_user_id_mock):
        get_current_user_id_mock.return_value = '123'
        with mock.patch('disco.gql.authentication.Authentication.query') as graphql_mock:
            graphql_mock.return_value = {'getArtifactsRepositories': [
                {'id': '5e143698be608b000c6064b0', 'displayName': 'Repository1', 'isActive': True},
                {'id': '5e143698be608b000c6064b1', 'displayName': 'Repository2', 'isActive': False},
                {'id': '5e143698be608b000c6064b2', 'displayName': 'Repository3', 'isActive': True}]}

            result = disco.Repository.list_repositories()

        assert isinstance(result, list)
        assert len(result) == 2
        assert result[0].id == '5e143698be608b000c6064b0'
        assert result[0].name == 'Repository1'
        assert result[1].id == '5e143698be608b000c6064b2'
        assert result[1].name == 'Repository3'





from tests.base_test import BaseTest
from mock import patch
import mock
import disco


class TestDockerImages(BaseTest):

    @BaseTest.with_config_and_env(authenticated=True)
    # @patch('disco.gql.authentication.Authentication.get_current_user')
    def test_list_docker_images(self):
        with mock.patch('disco.gql.authentication.Authentication.query') as graphql_mock:
            graphql_mock.return_value = {'getDockerImagesByOwnerId': [
                {'id': '5e143698be608b000c6064b0', 'displayName': 'docker 1', 'repositoryType': 'dockerHub',
                 'path': 'path1', 'entryPoint': 'ep1'},
                {'id': '5e143698be608b000c6064b1', 'displayName': 'docker 2', 'repositoryType': 'aws',
                 'path': 'path2', 'entryPoint': 'ep2'},
                {'id': '5e143698be608b000c6064b2', 'displayName': 'docker 3', 'repositoryType': 'gcp',
                 'path': 'path3', 'entryPoint': 'ep3'}]}

            result = disco.DockerImage.list_docker_images()

        assert isinstance(result, list)
        assert len(result) == 3
        assert result[0].id == '5e143698be608b000c6064b0'
        assert result[0].name == 'docker 1'
        assert result[0].repository_type == 'dockerHub'
        assert result[0].path == 'path1'
        assert result[0].entry_point == 'ep1'
        assert result[1].id == '5e143698be608b000c6064b1'
        assert result[1].name == 'docker 2'
        assert result[1].repository_type == 'aws'
        assert result[1].path == 'path2'
        assert result[1].entry_point == 'ep2'
        assert result[2].id == '5e143698be608b000c6064b2'
        assert result[2].name == 'docker 3'
        assert result[2].repository_type == 'gcp'
        assert result[2].path == 'path3'
        assert result[2].entry_point == 'ep3'





"""
Small utils shared between components
"""
import sys
import pkg_resources


def filter_nones(**kwargs):
    """Removes `None` parameters from kwargs

    Args:
        **kwargs:

    Returns:
        dict: `dict` of the non-null arguments
    """
    return {k: v for k, v in kwargs.items() if v is not None}


def pkg_version(package):
    """Gets the version of an installed (pip) package

    Args:
        package: the package name

    Returns:
        str: The version of the installed package.
    """
    try:
        dist = pkg_resources.get_distribution(package)
    except pkg_resources.DistributionNotFound:
        return None

    return str(dist.version)


def disco_version():
    """Gets disco's version"""
    return pkg_version('disco') or 'dev'


def str_without_prefix(data: str, prefix='/'):
    """Removes prefix (if present) from string.

    Args:
        data (str):
        prefix: prefix to be remove, defaults to '/'

    Returns:
        the string without the prefix if it was present
    """
    if data is None:
        return None
    return data if not data.startswith(prefix) else data[len(prefix):]


def str_without_suffix(data: str, suffix='/'):
    """Removes suffix (if present) from string.

    Args:
        data (str):
        suffix: suffix to be remove, defaults to '/'

    Returns:
        str: the string without the suffix if it was present
    """
    if data is None:
        return None
    return data if not data.endswith(suffix) else data[:-len(suffix)]


def is_venv():
    """Checks if the current code is executed in venv.
    Does NOT cover pyenv, conda and other setups

     Returns:
        bool: `True` if running venv, `False` otherwise
    """
    # https://stackoverflow.com/a/42580137/133568
    _has_real_prefix = hasattr(sys, 'real_prefix')
    _has_base_prefix = hasattr(sys, 'base_prefix')
    return _has_real_prefix or \
           (_has_base_prefix and sys.base_prefix != sys.prefix)


def shorten_string(data: str, max_len=15, short_tag: str = None):
    """Shortens thisverylongstring to thi..

    Args:
        data (str): input string
        max_len (int): max len of output
        short_tag (str): string to be used instead of shortened text

    Returns:
        str: the shortened text

    """
    short_tag = short_tag or '...'
    if not data or not isinstance(data, str) or len(data) <= max_len:
        return data

    return data[max_len - len(short_tag)] + short_tag


def merge_dictionaries(*args) -> dict:
    """Shallow merge dictionaries provided as arg

    Args:
        *args:

    Returns:
        dict: new instances of merged dictionaries
    """
    res = {}
    for arg in args:
        if arg is None or not isinstance(arg, dict):
            continue
        res.update(arg)
    return res



from mock import patch
from click.testing import CliRunner

from disco.core.exceptions import InvalidCredentials
from disco_cli import cli, setup_cli
from .cli_test_utils import output_message_includes

from tests.base_test import BaseTest


class TestAuthCommands(BaseTest):

    def setup_class(self):
        setup_cli()

    def test_login_success(self):
        """
        Tests a successful login
        Returns:

        """
        with patch('disco.gql.authentication.Authentication.login') as login_mock:
            runner = CliRunner()
            result = runner.invoke(cli, ['login', '-u', 'user@user.com', '-p', 'password'])
            assert result.exit_code == 0
            login_mock.assert_called_with('user@user.com', 'password')
            assert result.output == 'Signed in successfully\n'

    def test_login_fail(self):
        """
        Tests a login with wrong password
        Returns:

        """
        with patch('disco.gql.authentication.Authentication.login') as login_mock:
            login_mock.side_effect = InvalidCredentials("Wrong email or password")
            runner = CliRunner()
            result = runner.invoke(cli, ['login', '-u', 'user@user.com', '-p', 'password'])
            assert result.exit_code == 0
            assert result.output == 'Wrong username or password\n'

    def test_logout(self):
        """
        Tests logout
        """
        with patch('disco.gql.authentication.Authentication.logout') as logout_mock:
            runner = CliRunner()
            result = runner.invoke(cli, ['logout'])
            assert result.exit_code == 0
            logout_mock.assert_called()
            assert result.output == 'Logged out successfully\n'





#  Copyright (c) 2019 Samsung, Inc. and its affiliates.
#
#  This source code is licensed under the RESTRICTED license found in the
#  LICENSE file in the root directory of this source tree.

import io
import os
import pathlib
import uuid

import mock
import pytest
import requests

import disco
from .base_test import BaseTest
from disco import Asset
from contextlib import contextmanager

test_file_dir = os.path.dirname(os.path.realpath(__file__))
txt_file = os.path.join(test_file_dir, 'test_files', 'text_file.txt')
img_file = os.path.join(test_file_dir, 'test_files', 'disco-logo.png')

file_content = {}


class FTypes(object):
    text = 'Text'
    image = 'Image'


def setup_module():
    global file_content
    with io.FileIO(txt_file) as txt_f:
        file_content[FTypes.text] = txt_f.read()

    with io.FileIO(img_file) as img_f:
        file_content[FTypes.image] = img_f.read()


class TestAsset(BaseTest):

    def test__request_url_for_upload_with_cluster(self):
        file_name = self.random_str('filename')
        cluster_id = self.random_str('cluster_id')
        request_params = {
            'key': file_name,
            'clusterId': cluster_id
        }
        with mock.patch('disco.asset.Asset.rest') as fake_rest:
            Asset()._request_url_for_upload(file_name, cluster_id)
            fake_rest.assert_called_once_with(url='/files/uploadparams',
                                              data=request_params,
                                              method='post')

    def test__request_url_for_upload_without_cluster(self):
        file_name = self.random_str('filename')
        request_params = {
            'key': file_name,
        }
        with mock.patch('disco.asset.Asset.rest') as fake_rest:
            Asset()._request_url_for_upload(file_name, None)
            fake_rest.assert_called_once_with(url='/files/uploadparams',
                                              data=request_params,
                                              method='post')

    def test__register_file(self):
        token = self.random_str('token')
        send_result = self.random_str('id')

        with mock.patch('disco.asset.Asset.rest') as fake_rest:

            fake_rest.return_value = {'id': send_result}
            result = Asset()._register(token)
            fake_rest.assert_called_once_with(url='/files',
                                              data={'token': token},
                                              method='post')
            assert result == send_result

    file_argument_parameters = [
        (b'Lol I have text \n and newlines here.', FTypes.text),
        ('Lol I have text \n and newlines here.', FTypes.text),
        (io.StringIO('Lol I have text \n and newlines here.'), FTypes.text),
        (io.BytesIO(b'Lol I have text \n and newlines here.'), FTypes.text),
        (open(os.path.abspath(txt_file), 'r'), FTypes.text),
        (open(os.path.abspath(txt_file), 'rb'), FTypes.text),
        (pathlib.Path(txt_file), FTypes.text),
        (pathlib.Path(img_file), FTypes.image),
        (open(os.path.abspath(img_file), 'rb'), FTypes.image)
    ]
    @pytest.mark.parametrize('file_argument', file_argument_parameters)
    def test_upload_file_mocked(self, file_argument):
        generated_id = str(uuid.uuid4())
        generated_url = '/some-storage/%s' % generated_id
        generated_name = 'file_%s.bin' % generated_id
        generated_token = 'token>%s' % generated_id
        generated_fields = {'x': generated_id, 'y': generated_name}

        with mock.patch('disco.asset.Asset._request_url_for_upload') \
                as request_url_mock, \
                mock.patch('disco.asset.Asset._upload_with_callback') as _upload_with_callback, \
                mock.patch('disco.asset.Asset._register') as register_file_mock:

            request_url_mock.return_value = {
                'url': generated_url,
                'fields': generated_fields,
                'token': generated_token
            }

            register_file_mock.return_value = generated_id

            file_id = disco.asset.upload_file(generated_name, file_argument[0], show_progress_bar=False)

            request_url_mock.assert_called_once_with(generated_name, None)
            expected_body = file_content[file_argument[1]]

            _upload_with_callback.assert_called_once_with(
                generated_url, generated_name, form_fields=generated_fields,
                file_content_bytes=expected_body, callback=self.is_function())
            register_file_mock.assert_called_once_with(generated_token)

            assert file_id == generated_id

    def test_input_files_from_bucket(self):
        bucket_paths = self.random_list('bucket_path', length=3)
        cluster_id = self.random_str('clusterId')
        request_params = {'bucketPaths': bucket_paths, 'clusterId': cluster_id}
        fake_input_files = [dict(id=self.random_str('file_id')) for _ in range(3)]
        expected_input_file_ids = [fake_input_file['id'] for fake_input_file in fake_input_files]

        with mock.patch('disco.asset.Asset.rest') as fake_rest:
            fake_rest.return_value = fake_input_files

            result = Asset().input_files_from_bucket(bucket_paths, cluster_id)

            fake_rest.assert_called_once_with(url='/files/bucket', data=request_params, method='post')
            assert result == expected_input_file_ids

    def test_input_files_from_bucket_no_files_found(self):
        bucket_paths = self.random_list('bucket_path', length=3)
        cluster_id = self.random_str('clusterId')
        request_params = {'bucketPaths': bucket_paths, 'clusterId': cluster_id}
        empty_response = None

        with mock.patch('disco.asset.Asset.rest') as fake_rest:
            fake_rest.return_value = empty_response

            result = Asset().input_files_from_bucket(bucket_paths, cluster_id)

            fake_rest.assert_called_once_with(url='/files/bucket', data=request_params, method='post')
            assert result == []



from tests.base_test import BaseTest
from disco.models import Task


class TestTaskModel(BaseTest):

    def test_task_create(self):

        task_id = self.random_str('task_id')
        task_status = 'Success'
        task_duration = self.random_int()
        task_input_file_id = self.random_str('input_file_id')
        artifact_ids = [self.random_str('artifact_id1'), self.random_str('artifact_id2')]

        task_data = {
            'id': task_id,
            'status': task_status,
            'stats': {'duration': task_duration},
            'request': {'inputFile': {'id': task_input_file_id}},
            'result': {'artifactIds': artifact_ids}
        }

        task = Task(task_data)
        assert task.id == task_id
        assert task.status == task_status
        assert task.duration == task_duration
        assert task.input_file_id == task_input_file_id
        assert task.artifact_ids == artifact_ids

    def test_task_create_without_artifact_ids(self):

        task_id = self.random_str('task_id')
        task_status = 'Success'

        task_data = {
            'id': task_id,
            'status': task_status,
        }

        task = Task(task_data)
        assert task.id == task_id
        assert task.status == task_status
        assert task.artifact_ids == []



"""
Upload a file to DISCO, so it could later be used to run jobs.
"""
import io
import os
import pathlib
import requests
from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor

from disco.core import create_progress_bar, constants
from .base_controller import BaseController


class Asset(BaseController):
    """Provides functionality for uploading and downloading disco files"""

    def _request_url_for_upload(self, file_name, cluster_id=None):
        params = {'key': file_name}
        if cluster_id:
            params['clusterId'] = cluster_id
        return self.rest(url='/files/uploadparams',
                         data=params,
                         method='post'
                         )

    @classmethod
    def _upload_with_callback(cls, url, file_name, form_fields, file_content_bytes, callback):
        """
        Upload file with callback for monitoring the progress

        Args:
            url (str):
            file_name (str):
            form_fields (dict):
            file_content_bytes (bytes):
            callback (func):
        """
        encoder = MultipartEncoder({
            **form_fields,
            'file': (file_name, io.BytesIO(file_content_bytes)),
        })
        monitor = MultipartEncoderMonitor(encoder, callback)
        response = requests.request(
            'post',
            url,
            data=monitor,
            headers={'Content-Type': monitor.content_type}
        )
        response.raise_for_status()

    def _register(self, token):
        return self.rest(
            url='/files',
            data={'token': token},
            method='post')['id']

    def upload(self, file_name, file, cluster_id=None, show_progress_bar=True):
        """Upload a file to DISCO, so it could later be used to run jobs.

        Args:
            file_name (str):
            file: `file` can be either the file contents,
                in binary or string forms, a file
                object, or a Path` object that points to a file.
            cluster_id (str):
            show_progress_bar (bool):

        Returns:
            str: The id of the uploaded file.
        """
        if isinstance(file, bytes):
            file_content = file
        elif isinstance(file, str):
            file_content = file.encode()
        elif isinstance(file, pathlib.Path):
            file_content = file.read_bytes()
        elif hasattr(file, 'read'):
            file_content = file.read()
            if isinstance(file_content, bytes):
                pass
            elif isinstance(file_content, str):
                file_content = file_content.encode()
        else:
            file_content = pathlib.Path(str(file)).read_bytes()

        data = self._request_url_for_upload(file_name, cluster_id)

        disable_progress_bar = os.environ.get(constants.ENV_VAR_DISABLE_PROGRESS_BAR) == '1' or (not show_progress_bar)

        with create_progress_bar(
                total=len(file_content),
                desc=f"Uploading {file_name}", unit='B',
                ncols=80, disable=disable_progress_bar, unit_scale=True) as progress_bar:

            update_progress_bar_callback = self.__create_update_progress_bar_callback(progress_bar)

            self._upload_with_callback(data['url'],
                                       file_name,
                                       form_fields=data['fields'],
                                       file_content_bytes=file_content,
                                       callback=update_progress_bar_callback)

        register_result = self._register(data['token'])
        return register_result

    @staticmethod
    def __create_update_progress_bar_callback(progress_bar):
        context = dict(bytes_read_so_far=0)

        def update_progress_bar_callback(monitor):
            new_bytes_read = monitor.bytes_read
            current_delta = new_bytes_read - context['bytes_read_so_far']
            context['bytes_read_so_far'] = new_bytes_read
            progress_bar.update(current_delta)

        return update_progress_bar_callback

    def input_files_from_bucket(self, bucket_paths, cluster_id):
        """
        Args:
            bucket_paths (list(str)):
            cluster_id (str):

        Raises:
            BucketPathsException: In case the bucket path is invalid or missing or has no files

        Returns:
            list - List of files ids registered from the given bucket paths
        """
        input_file_ids = self._request_register_files_from_bucket(bucket_paths, cluster_id)

        return input_file_ids

    def _request_register_files_from_bucket(self, bucket_paths, cluster_id):
        """
        Args:
            bucket_paths (list(str)):
        cluster_id (str):

        Returns:
            list - List of files ids registered from the given bucket paths
        """
        params = {'bucketPaths': bucket_paths, 'clusterId': cluster_id}
        registered_input_files = self.rest(url='/files/bucket', data=params, method='post')

        if registered_input_files is None:
            return []

        return [input_file.get('id') for input_file in registered_input_files]


def upload_file(file_name, file, cluster_id=None, show_progress_bar=True):
    """Legacy: Uploads file to DISCO, see `Assest.upload` for more info.

    Args:
        file_name (str):
        file: `file` can be either the file contents,
                in binary or string forms, a file
                object, or a Path` object that points to a file.
        cluster_id (str):
        show_progress_bar (bool):

    Returns:
        str: The id of the uploaded file.
    """
    return Asset().upload(file_name, file, cluster_id=cluster_id, show_progress_bar=show_progress_bar)


def input_files_from_bucket(bucket_paths, cluster_id):
    """
    Will scan the provided bucket paths for input files to be used in a job

    Args:
        bucket_paths (list(str)): List of paths in a bucket, can include wildcards
        cluster_id (str): A cluster id with a bucket to scan

    Raises:
        BucketPathsException: In case the bucket path is invalid or missing or has no files

    Returns:
        list - List of files ids registered from the given bucket paths
    """
    return Asset().input_files_from_bucket(bucket_paths, cluster_id)



import click

from cli.command_utils import info_message, list_to_table
from disco import Cluster

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group('cluster', context_settings=CONTEXT_SETTINGS)
def cluster_commands():
    """Manage clusters."""


@cluster_commands.command('list', context_settings=CONTEXT_SETTINGS,
                          short_help="List available clusters for the current user.")
def list_clusters():
    """
    List available clusters for the current user.

    $ disco cluster list
    """
    clusters = Cluster.list_clusters()
    display_clusters(clusters)


def display_clusters(clusters_list):
    """
    Display clusters as table
    Args:
        clusters_list (list): list of cluster details
    """
    cluster_table = []
    for cluster in clusters_list:
        cluster_table.append(
            [
                "V" if cluster.is_default else "",
                cluster.id,
                cluster.name,
                cluster.type,
                cluster.region,
                cluster.external_account_id,
                cluster.bucket_name,
                "Yes" if cluster.is_last_connection_check_valid else "No",
                "Yes" if cluster.is_active else "No",
                "Yes" if cluster.cloud_offload_enabled else "No"
            ]
        )
    info_message(list_to_table(cluster_table, headers=["Default", "ID", "Name", "Cloud type", "Region",
                                                       "Account", "Bucket", "Connection valid", "Active",
                                                       "Cloud offload enabled"]))



import click
from disco.core.exceptions import InvalidCredentials, DiscoException
from disco.gql import Authentication
from .command_utils import success_message, error_message

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.command('login', context_settings=CONTEXT_SETTINGS)
@click.option('-u', '--user', help='User email to log in with.', prompt='Enter username')
@click.option('-p', '--password', help='User password to log in with.', prompt='Enter password', hide_input=True)
def login(user, password):
    """
    Login to Dis.co.

    $ disco login -u <email> -p <password>
    """
    try:
        Authentication.login(user, password)
        success_message("Signed in successfully")
    except InvalidCredentials:
        error_message("Wrong username or password")
    except DiscoException:
        error_message("Unknown error occurred")


@click.command('logout', context_settings=CONTEXT_SETTINGS)
def logout():
    """
    Logout of Dis.co.

    $ disco logout
    """
    Authentication.logout()
    success_message("Logged out successfully")



import os
from requests import HTTPError, ConnectionError as RequestsConnectionError
from tabulate import tabulate
import click

from disco.core.exceptions import DiscoException, GraphQLRequestException, InvalidCredentials
from disco.gql import Authentication


class ExpandedPath(click.Path):
    """
    Allows click.Path() validator to convert user's home path
    i.e `~/some_file.py` => `/Users/SOME_USER/some_file.py`

    see:
        https://github.com/pallets/click/issues/287
    """
    def convert(self, value, param, ctx):
        value = os.path.expanduser(value)
        return super(ExpandedPath, self).convert(value, param, ctx)


def verify_logged_in():
    """
    Verifies that the user is logged in and print error message if not
    Returns: True if user is logged in

    """
    if not Authentication.is_logged_in():
        error_message("You must be logged in to perform this operation")
        return False
    return True


def success_message_func(message):
    """
    A helper function for creating success message functions

    Args:
        message (str): The message to show

    Returns: a function that outputs a success message

    """

    return lambda response=None: click.echo(click.style(message, fg='green'))


def error_message_func(message):
    """
    A helper function for creating error message functions

    Args:
        message (str): The message to show

    Returns: a function that outputs an error message

    """

    return lambda response=None: click.echo(click.style(message, fg='red'))


def exception_message(exception):
    """
        A helper function for creating error messages
        The function determines the message to show by the exception given

        Args:
            message (str): The message to show

        Returns: a function that outputs an error message

        """
    click.echo(click.style(f"{get_error_message(exception)}", fg='red'))


def get_error_message(exception):
    """
    Get a message for the given exception
    Args:
        exception (:obj:`Exception`): An exception

    Returns:
        A message for the user according to the exception
    """
    if isinstance(exception, DiscoException):
        if isinstance(exception, GraphQLRequestException):
            messages = list(map(lambda error: error['message'], exception.errors))
            if any(msg.find('Cast to ObjectId failed') >= 0 for msg in messages):
                return "Bad format for Id"
        if isinstance(exception, InvalidCredentials):
            return "Wrong username or password - run 'disco login' and try again"

    if isinstance(exception, RequestsConnectionError):
        return "No internet connection"

    if isinstance(exception, HTTPError):
        if exception.response.status_code == 400:
            return "Not logged in or bad request"

    return 'Unknown'


def error_message(message):
    """
    Outputs an error message
    Args:
        message (str): Message to show

    Returns:
        None
    """
    error_message_func(message)()


def success_message(message):
    """
    Outputs a success message
    Args:
        message (str): Message to show

    Returns:
        None
    """
    success_message_func(message)()


def info_message(message):
    """
    Outputs an informational message
    Args:
        message (str): Message to show

    Returns:
        None
    """
    click.echo(click.style(message, fg='yellow'))


def list_to_table(obj_list, headers=""):
    """
    Converts a list to a displayable table
    The list must be a list of lists with the same length
    Args:
        obj_list (:obj:`list` of :obj:`list` of :obj:`str`): List of objects to display
        headers: (:obj:`list` of :obj:`str): Table headers

    Returns:
        A string that contains a table
    """
    return tabulate(obj_list, headers, tablefmt='grid')



"""
Exceptions raised by the SDK
"""


class _ExceptionWithDocSupport(Exception):
    """Base exception that uses its first docstring line in lieu of a
    message.
    """

    def __init__(self, message=None):
        # We use `None` as the default for `message`, so the user can input ''
        # to force an empty message.

        """
        Args:
            message:
        """
        if message is None:
            if isinstance(self, _ExceptionWithDocSupport):
                message = (self.__doc__ or '').strip().split('\n')[0]

        super().__init__(message)


class DiscoException(_ExceptionWithDocSupport):
    """The base of all other exceptions"""


class NoCredentials(DiscoException):
    """Please provide email and password for DISCO,
    use `disco.set_credentials`.
    """


class InvalidCredentials(DiscoException):
    """
    Wrong email or password
    """


class NoValidProfileException(DiscoException):
    """This user is not allowed to run jobs on Dis.co"""


class TimeOutError(DiscoException, TimeoutError):  # pylint: disable=too-many-ancestors
    """Timeout error"""


class RequestException(DiscoException):
    """Error in a request sent to the backend of DISCO"""

    DEFAULT_MESSAGE = 'Request error'

    def __init__(self, message=None, additional_info=None):
        super().__init__(message or self.DEFAULT_MESSAGE)
        self.additional_info = additional_info or {}


class GraphQLRequestException(DiscoException):
    """
    Error in GraphQL call
    """

    errors = []

    def __init__(self, message=None, graphql_errors=None):
        self.errors = graphql_errors if graphql_errors is not None else []
        super().__init__(message)

    def messages(self):
        """Returns all messages from the gql errors"""
        return list(map(lambda err: err['message'], self.errors))


class BucketPathsException(DiscoException):
    """
    User got an error when trying to register files from a bucket path/s
    """

    def __init__(self, message=None, bucket_paths_errors=None):
        super().__init__(message)
        self.bucket_paths_errors = bucket_paths_errors or {}


class DiscoErrorCodes:
    """
    Disco Error Codes
    """
    BucketPaths = 'bucket_paths'


class BucketPathsErrorTypes:
    """
    Bucket Paths Error Types
    """
    # User tried to register files from an invalid or missing bucket path
    InvalidPath = 'invalid_path'

    # User tried to register files from an empty bucket path
    NoFilesInPath = 'no_files_in_path'



import click
import cli.utilities.utils as utils


CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.command('version', context_settings=CONTEXT_SETTINGS)
def cli_version():
    """
    Show the installed Dis.co CLI version.

    $ disco version
    """
    click.echo(f'Dis.co Command Line version {utils.disco_cli_version()} is installed')



#  Copyright (c) 2019 Samsung, Inc. and its affiliates.
#
#  This source code is licensed under the RESTRICTED license found in the
#  LICENSE file in the root directory of this source tree.

import setuptools
import sys


if sys.version_info < (3, 6):
    raise RuntimeError("This package requires Python 3.6+")

setuptools.setup(
    name='disco',
	version="1.23.0",
    author='Disco',
    author_email='contact@dis.co',
    description='Run Python jobs on the cloud using DISCO',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/Iqoqo/python-sdk',
    py_modules=['disco_cli'],
    packages=setuptools.find_packages(),
    install_requires=open('requirements.txt', 'r').read().split('\n'),
    python_requires='>=3.6',
    tests_require=open('test_requirements.txt', 'r').read().split('\n'),
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Operating System :: OS Independent',
    ],
    package_data={
        '': ['*.txt', '*.rst', '*.gql'],
        'disco.gql.docker.commands': ['*']
    },
    entry_points='''
            [console_scripts]
            disco=disco_cli:main
        ''',
)



import click

from cli.command_utils import info_message, list_to_table
from disco import DockerImage

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(context_settings=CONTEXT_SETTINGS)
def docker():
    """
    Manage docker images.
    """


@docker.command('list', context_settings=CONTEXT_SETTINGS, short_help="List of docker images of the current user.")
def list_dockers():
    """
    List of docker images of the current user.

    $ disco docker list
    """
    docker_images = DockerImage.list_docker_images()
    if len(docker_images) > 0:
        _display_docker_images(docker_images)
    else:
        info_message("There are no set docker images for your account")


def _display_docker_images(docker_images_list):
    """
    Display docker images as table
    Args:
        docker_images_list: list of docker images details

    Returns:
    """
    docker_images_table = [[docker_img.id, docker_img.name, docker_img.repository_type, docker_img.path,
                            docker_img.entry_point] for docker_img in docker_images_list]
    info_message(list_to_table(docker_images_table, headers=["ID", "Name", "Repository Type", "Path", "Entry Point"]))
